--------------------------------------------------------------------------------
-- par_framer_mult.vhd
--
-- SMPTE 259M-1997 Parallel Framer Module
--
--
--
--                  Author: John F. Snow
--                  Staff Applications Engineer
--
--                  Video Applications
--                  Advanced Products Group
--                  Xilinx, Inc.
--
--                  Copyright (c) 2001 Xilinx, Inc.
--                  All rights reserved
--
--                  Date:   Aug. 17, 2001
--
--                  RESTRICTED RIGHTS LEGEND
--
--      This software has not been published by the author, and 
--      has been disclosed to others for the purpose of enhancing 
--      and promoting design productivity in Xilinx products.
--
--      Therefore use, duplication or disclosure, now and in the 
--      future should give consideration to the productivity 
--      enhancements afforded the user of this code by the author's 
--      efforts.  Thank you for using our products !
--
-- Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
-- Revision:
--          Aug. 17, 2001     Translated from Verilog
--          Oct. 22, 2003     Updated the std_logic_vector <=> std_ulogic_vector
--                            conversion to be more compatible
--
--
-- Other modules instanced in this design:
--
--          None.
--------------------------------------------------------------------------------
-- 
-- Description of module:
-- 
-- SMPTE 259M-1997 serial digital interface (SDI) is a standard for transmitting
-- digital video over a serial link.
-- 
-- This module accepts 10-bit parallel "unframed" data words from the SDI
-- descrambler module and examines the bit stream for the 30-bit TRS pramble. 
-- Once a TRS is found, the framer then knows the bit boundary of all subsequent
-- 10-bit characters in the bit stream and uses this offset to generate a 10-bit
-- parallel data word on its output.
-- 
-- The module has the following control inputs:
-- 
-- ce: The clock enable input controls loading of all registers in the module. 
-- It must be asserted whenever a new 10-bit word is to be loaded into the 
-- module. By providing a clock enable, this module can use a clock that is 
-- running at the bit rate of the SDI bit stream if ce is asserted once every 
-- ten clock cycles.
-- 
-- frame_en: This input controls whether the framer rsynchronize to new 
-- character offsets when out-of-phase TRS symbols are detected. When this input
-- is high, out-of-phase TRS symbols will cause the framer to resynchronize.
-- 
-- The module generates the following outputs:
-- 
-- q: This 10-bit output port contains the properly framed data word. The data 
-- is normally valid on this port for 10 clock cycles.
-- 
-- trs: (timing reference signal) This output is asserted when the q output
-- register holds any of the four words of a TRS.
-- 
-- nsp: (new start position) If frame_en is low and a TRS is detected that does 
-- not match the current character offset, this signal will be asserted high. 
-- The nsp signal will remain asserted until the offset error has been 
-- corrected.
-- 
-- There are normally three ways to use the frame_en input:
-- 
-- frame_en tied high: When frame_en is tied high, the framer will resynchronize
-- on every TRS detected. 
-- 
-- frame_en tied to nsp: When in this mode, the framer implements TRS filtering.
-- If a TRS is detected that is out of phase with the existing character offset,
-- nsp will be asserted, but the framer will not resynchronize. If the next TRS
-- received is in phase with the current character offset, nsp will go low and 
-- the will not resynchronize. If the next TRS arrives out of phase with the 
-- current character offset, then the new character offset will be loaded and 
-- nsp will be deasserted. Single erroneous TRS  are ignored in this mode, but 
-- if they persist, the decoder will adjust.
-- 
-- frame_en tied low: The automatic framing function is disabled when frame_en 
-- is tied low. If data is being sent across the interface that does not comply 
-- with the SDI standard and may contain data that looks like TRS symbols, the 
-- framing function can be disabled in this manner.
--
-- This version of the parallel framer uses a Virtex-II embedded multiplier to
-- implement the barrel shifter.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all ;

entity par_framer_mult is
    port (
        clk:        in  std_ulogic;                    -- clock input
        rst:        in  std_ulogic;                    -- async reset input
        ce:         in  std_ulogic;                    -- clock enable input
        d:          in  std_ulogic_vector(9 downto 0); -- input data bus
        frame_en:   in  std_ulogic;                    -- enables resynch when 1
        q:          out std_ulogic_vector(9 downto 0); -- output data bus
        trs:        out std_ulogic;                    -- 1 when q contains TRS
        nsp:        out std_ulogic                     -- new start position
    );
end par_framer_mult;

architecture synth of par_framer_mult is
    
-- Internal signals
signal in1_reg:         std_ulogic_vector( 9 downto 0); -- input register 1
signal in2_reg:         std_ulogic_vector( 9 downto 0); -- input register 2
signal in3_reg:         std_ulogic_vector( 9 downto 0); -- input register 3
signal offset_reg:      std_ulogic_vector( 9 downto 0); -- offset register
signal barrel_in:       std_ulogic_vector(18 downto 0); -- input reg for barrel shifter
signal trs_out:         std_ulogic_vector( 3 downto 0); -- used to generate trs signal
signal barrel_in_s :    std_logic_vector (17 downto 0);
signal in_vector:       std_ulogic_vector(38 downto 0); -- concat of 3 input regs & d
signal trs_match1:      std_ulogic_vector( 9 downto 0); -- offsets in in_vector(18:0) matching 0x3ff
signal trs_match2:      std_ulogic_vector( 9 downto 0); -- offsets in in_vector(28:10) matching 0x000
signal trs_match3:      std_ulogic_vector( 9 downto 0); -- offsets in in_vector(38:20) matching 0x000
signal trs_match_all:   std_ulogic_vector( 9 downto 0); -- offsets matching complete TRS preamble
signal trs_match1_l1:   std_ulogic_vector(15 downto 0); -- intermediate level of gate outputs
signal trs_match2_l1:   std_ulogic_vector(15 downto 0); -- intermediate level of gate outputs
signal trs_match3_l1:   std_ulogic_vector(15 downto 0); -- intermediate level of gate outputs
signal trs_detected:    std_ulogic;                     -- asserted when TRS is detected
signal trs_error:       std_ulogic;                     -- more than one offset matched the TRS symbol
signal offset_val:      std_ulogic_vector( 9 downto 0); -- calculated TRS offset value
signal barrel_out:      std_logic_vector (35 downto 0); -- output of barrel shifter
signal new_offset:      std_ulogic;                     -- mismatch between offset_val and offset_reg
signal barrel_msb:      std_ulogic;                     -- msb of barrel shifter
signal sc:              std_logic_vector (17 downto 0);
signal barrel_out_u:    std_ulogic_vector(8 downto 0);  -- useful portion of MULT output converted to std_ulogic_vector

-- Component declarations
component MULT18X18
    port(
        A: in  std_logic_vector(17 downto 0);
        B: in  std_logic_vector(17 downto 0);
        P: out std_logic_vector(35 downto 0)
    );
end component;

begin
        
    --
    -- Input registers 1, 2, and 3
    --
    -- The following three registers form a 3-character deep pipeline
    -- which will be examined by the 39-bit wide TRS detector.
    --
    process(clk, rst)
    begin
        if (rst = '1') then
            in1_reg <= (others => '0');
            in2_reg <= (others => '0');
            in3_reg <= (others => '0');
        elsif (clk'event and clk='1') then
            if (ce = '1') then
                in1_reg <= d;
                in2_reg <= in1_reg;
                in3_reg <= in2_reg;
            end if;
        end if;
    end process;    

    --
    -- TRS detector
    --
    -- The TRS detector finds 30-bit TRS preambles (0x3ff, 0x000, 0x000) in the
    -- input data stream. The TRS detector scans a 39-bit input vector
    -- consisting of all the bits from the three input registers plus the LS
    -- 9 bits of the d input data.
    --
    -- The detector consists two main parts. 
    --
    -- The first part is a series 10-bit AND and NOR gates that examine each
    -- possible bit location in the 39 input vector for the TRS preamble. These
    -- 10-bit wide AND and NOR gates have been coded here as two levels of
    -- 3 and 4 input gates because this results in a more compact implementation
    -- in most synthesis engines. 
    --
    -- The outputs of these gates are assigned to the vectors trs_match1, 2, 
    -- and 3. These three vectors each contain 10 unary bits that indicate which 
    -- offset(s) matched the pattern being detected. ANDing these three vectors
    -- together generates another 10-bit vector called trs_match_all whose bits
    -- indicate which offset(s) matches the entire 30-bit TRS preamble.
    --
    -- The second part of the TRS detector determines if one and only one of the
    -- bits in the trs_match_all vector are set. If at least one bit is set, 
    -- then the trs_detected signal is asserted. If more than one bit is set, 
    -- then the trs_error signal is also asserted.
    -- 
    in_vector <= d(8 downto 0) & in1_reg & in2_reg & in3_reg;

    -- first level of gates

    process(in_vector)
    begin
        trs_match1_l1( 0) <= in_vector( 3) and in_vector( 2) and 
                             in_vector( 1) and in_vector( 0);
        trs_match1_l1( 1) <= in_vector( 4) and in_vector( 3) and 
                             in_vector( 2) and in_vector( 1);
        trs_match1_l1( 2) <= in_vector( 5) and in_vector( 4) and 
                             in_vector( 3) and in_vector( 2);
        trs_match1_l1( 3) <= in_vector( 6) and in_vector( 5) and 
                             in_vector( 4) and in_vector( 3);
        trs_match1_l1( 4) <= in_vector( 7) and in_vector( 6) and 
                             in_vector( 5) and in_vector( 4);
        trs_match1_l1( 5) <= in_vector( 8) and in_vector( 7) and 
                             in_vector( 6) and in_vector( 5);
        trs_match1_l1( 6) <= in_vector( 9) and in_vector( 8) and 
                             in_vector( 7) and in_vector( 6);
        trs_match1_l1( 7) <= in_vector(10) and in_vector( 9) and 
                             in_vector( 8) and in_vector( 7);
        trs_match1_l1( 8) <= in_vector(11) and in_vector(10) and 
                             in_vector( 9) and in_vector( 8);
        trs_match1_l1( 9) <= in_vector(12) and in_vector(11) and 
                             in_vector(10) and in_vector( 9);
        trs_match1_l1(10) <= in_vector(13) and in_vector(12) and 
                             in_vector(11) and in_vector(10);
        trs_match1_l1(11) <= in_vector(14) and in_vector(13) and 
                             in_vector(12) and in_vector(11);
        trs_match1_l1(12) <= in_vector(15) and in_vector(14) and 
                             in_vector(13) and in_vector(12);
        trs_match1_l1(13) <= in_vector(16) and in_vector(15) and 
                             in_vector(14) and in_vector(13);
        trs_match1_l1(14) <= in_vector(17) and in_vector(16) and 
                             in_vector(15) and in_vector(14);
        trs_match1_l1(15) <= in_vector(18) and in_vector(17) and 
                             in_vector(16) and in_vector(15);

        trs_match2_l1( 0) <= not (in_vector(13) or in_vector(12) or 
                                  in_vector(11) or in_vector(10));
        trs_match2_l1( 1) <= not (in_vector(14) or in_vector(13) or 
                                  in_vector(12) or in_vector(11));
        trs_match2_l1( 2) <= not (in_vector(15) or in_vector(14) or 
                                  in_vector(13) or in_vector(12));
        trs_match2_l1( 3) <= not (in_vector(16) or in_vector(15) or 
                                  in_vector(14) or in_vector(13));
        trs_match2_l1( 4) <= not (in_vector(17) or in_vector(16) or 
                                  in_vector(15) or in_vector(14));
        trs_match2_l1( 5) <= not (in_vector(18) or in_vector(17) or 
                                  in_vector(16) or in_vector(15));
        trs_match2_l1( 6) <= not (in_vector(19) or in_vector(18) or 
                                  in_vector(17) or in_vector(16));
        trs_match2_l1( 7) <= not (in_vector(20) or in_vector(19) or 
                                  in_vector(18) or in_vector(17));
        trs_match2_l1( 8) <= not (in_vector(21) or in_vector(20) or 
                                  in_vector(19) or in_vector(18));
        trs_match2_l1( 9) <= not (in_vector(22) or in_vector(21) or 
                                  in_vector(20) or in_vector(19));
        trs_match2_l1(10) <= not (in_vector(23) or in_vector(22) or 
                                  in_vector(21) or in_vector(20));
        trs_match2_l1(11) <= not (in_vector(24) or in_vector(23) or 
                                  in_vector(22) or in_vector(21));
        trs_match2_l1(12) <= not (in_vector(25) or in_vector(24) or 
                                  in_vector(23) or in_vector(22));
        trs_match2_l1(13) <= not (in_vector(26) or in_vector(25) or 
                                  in_vector(24) or in_vector(23));
        trs_match2_l1(14) <= not (in_vector(27) or in_vector(26) or 
                                  in_vector(25) or in_vector(24));
        trs_match2_l1(15) <= not (in_vector(28) or in_vector(27) or 
                                  in_vector(26) or in_vector(25));

        trs_match3_l1( 0) <= not (in_vector(23) or in_vector(22) or 
                                  in_vector(21) or in_vector(20));
        trs_match3_l1( 1) <= not (in_vector(24) or in_vector(23) or 
                                  in_vector(22) or in_vector(21));
        trs_match3_l1( 2) <= not (in_vector(25) or in_vector(24) or 
                                  in_vector(23) or in_vector(22));
        trs_match3_l1( 3) <= not (in_vector(26) or in_vector(25) or 
                                  in_vector(24) or in_vector(23));
        trs_match3_l1( 4) <= not (in_vector(27) or in_vector(26) or 
                                  in_vector(25) or in_vector(24));
        trs_match3_l1( 5) <= not (in_vector(28) or in_vector(27) or 
                                  in_vector(26) or in_vector(25));
        trs_match3_l1( 6) <= not (in_vector(29) or in_vector(28) or 
                                  in_vector(27) or in_vector(26));
        trs_match3_l1( 7) <= not (in_vector(30) or in_vector(29) or 
                                  in_vector(28) or in_vector(27));
        trs_match3_l1( 8) <= not (in_vector(31) or in_vector(30) or 
                                  in_vector(29) or in_vector(28));
        trs_match3_l1( 9) <= not (in_vector(32) or in_vector(31) or 
                                  in_vector(30) or in_vector(29));
        trs_match3_l1(10) <= not (in_vector(33) or in_vector(32) or 
                                  in_vector(31) or in_vector(30));
        trs_match3_l1(11) <= not (in_vector(34) or in_vector(33) or 
                                  in_vector(32) or in_vector(31));
        trs_match3_l1(12) <= not (in_vector(35) or in_vector(34) or 
                                  in_vector(33) or in_vector(32));
        trs_match3_l1(13) <= not (in_vector(36) or in_vector(35) or 
                                  in_vector(34) or in_vector(33));
        trs_match3_l1(14) <= not (in_vector(37) or in_vector(36) or 
                                  in_vector(35) or in_vector(34));
        trs_match3_l1(15) <= not (in_vector(38) or in_vector(37) or 
                                  in_vector(36) or in_vector(35));
    end process;

    -- second level of gates

    process(trs_match1_l1)
    begin
        trs_match1(0) <= trs_match1_l1( 0) and trs_match1_l1( 4) and 
                         trs_match1_l1( 6);
        trs_match1(1) <= trs_match1_l1( 1) and trs_match1_l1( 5) and 
                         trs_match1_l1( 7);
        trs_match1(2) <= trs_match1_l1( 2) and trs_match1_l1( 6) and 
                         trs_match1_l1( 8);
        trs_match1(3) <= trs_match1_l1( 3) and trs_match1_l1( 7) and 
                         trs_match1_l1( 9);
        trs_match1(4) <= trs_match1_l1( 4) and trs_match1_l1( 8) and 
                         trs_match1_l1(10);
        trs_match1(5) <= trs_match1_l1( 5) and trs_match1_l1( 9) and 
                         trs_match1_l1(11);
        trs_match1(6) <= trs_match1_l1( 6) and trs_match1_l1(10) and 
                         trs_match1_l1(12);
        trs_match1(7) <= trs_match1_l1( 7) and trs_match1_l1(11) and 
                         trs_match1_l1(13);
        trs_match1(8) <= trs_match1_l1( 8) and trs_match1_l1(12) and 
                         trs_match1_l1(14);
        trs_match1(9) <= trs_match1_l1( 9) and trs_match1_l1(13) and 
                         trs_match1_l1(15);
    end process;

    process(trs_match2_l1)
    begin
        trs_match2(0) <= trs_match2_l1( 0) and trs_match2_l1( 4) and 
                         trs_match2_l1( 6);
        trs_match2(1) <= trs_match2_l1( 1) and trs_match2_l1( 5) and 
                         trs_match2_l1( 7);
        trs_match2(2) <= trs_match2_l1( 2) and trs_match2_l1( 6) and 
                         trs_match2_l1( 8);
        trs_match2(3) <= trs_match2_l1( 3) and trs_match2_l1( 7) and 
                         trs_match2_l1( 9);
        trs_match2(4) <= trs_match2_l1( 4) and trs_match2_l1( 8) and 
                         trs_match2_l1(10);
        trs_match2(5) <= trs_match2_l1( 5) and trs_match2_l1( 9) and 
                         trs_match2_l1(11);
        trs_match2(6) <= trs_match2_l1( 6) and trs_match2_l1(10) and 
                         trs_match2_l1(12);
        trs_match2(7) <= trs_match2_l1( 7) and trs_match2_l1(11) and 
                         trs_match2_l1(13);
        trs_match2(8) <= trs_match2_l1( 8) and trs_match2_l1(12) and 
                         trs_match2_l1(14);
        trs_match2(9) <= trs_match2_l1( 9) and trs_match2_l1(13) and 
                         trs_match2_l1(15);
    end process;

    process(trs_match3_l1)
    begin
        trs_match3(0) <= trs_match3_l1( 0) and trs_match3_l1( 4) and 
                         trs_match3_l1( 6);
        trs_match3(1) <= trs_match3_l1( 1) and trs_match3_l1( 5) and 
                         trs_match3_l1( 7);
        trs_match3(2) <= trs_match3_l1( 2) and trs_match3_l1( 6) and 
                         trs_match3_l1( 8);
        trs_match3(3) <= trs_match3_l1( 3) and trs_match3_l1( 7) and 
                         trs_match3_l1( 9);
        trs_match3(4) <= trs_match3_l1( 4) and trs_match3_l1( 8) and 
                         trs_match3_l1(10);
        trs_match3(5) <= trs_match3_l1( 5) and trs_match3_l1( 9) and 
                         trs_match3_l1(11);
        trs_match3(6) <= trs_match3_l1( 6) and trs_match3_l1(10) and 
                         trs_match3_l1(12);
        trs_match3(7) <= trs_match3_l1( 7) and trs_match3_l1(11) and 
                         trs_match3_l1(13);
        trs_match3(8) <= trs_match3_l1( 8) and trs_match3_l1(12) and 
                         trs_match3_l1(14);
        trs_match3(9) <= trs_match3_l1( 9) and trs_match3_l1(13) and 
                         trs_match3_l1(15);
    end process;

    -- third level of gates generates a unary bit pattern indicating which 
    -- offsets contain valid TRS symbols
    trs_match_all <= trs_match1 and trs_match2 and trs_match3;

    -- If any of the bits in trs_match_all are asserted, the assert trs_detected        
    trs_detected <= '0' when (trs_match_all = (trs_match_all'range => '0')) else
                    '1';

    -- The case statement determines if more than one bit is set, asserting trs_error
    -- if so. The for loop generates the offset_val vector by reversing the bit
    -- order of trs_match_all.
    process(trs_match_all)
    begin
        case trs_match_all is
            when "0000000000" => trs_error  <= '0';
            when "0000000001" => trs_error  <= '0';
            when "0000000010" => trs_error  <= '0';
            when "0000000100" => trs_error  <= '0';
            when "0000001000" => trs_error  <= '0';
            when "0000010000" => trs_error  <= '0';
            when "0000100000" => trs_error  <= '0';
            when "0001000000" => trs_error  <= '0';
            when "0010000000" => trs_error  <= '0';
            when "0100000000" => trs_error  <= '0';
            when "1000000000" => trs_error  <= '0';
            when others       => trs_error  <= '1';
        end case;   

        for i in 0 to 9 loop
            offset_val(i) <= trs_match_all(9 - i);
        end loop;
    end process;

    --
    -- offset_reg: barrel shifter offset register
    --
    -- The offset_reg loads the offset_val whenever trs_detected is
    -- asserted and trs_error is not asserted and frame_en is asserted.
    --
    process(clk, rst)
    begin
        if (rst = '1') then
            offset_reg <= (others => '0');
        elsif (clk'event and clk = '1') then
            if (ce = '1') then
                if (trs_detected = '1' and trs_error = '0' and 
                    frame_en = '1') then
                    offset_reg <= offset_val;
                end if;
            end if;
        end if;
    end process;

    --
    -- New start position detector
    -- 
    -- A comparison between offset_val and offset_reg determines if
    -- the new offset is different than the current one. If there is
    -- a mismatch and frame_en is not asserted, then the nsp output
    -- will be asserted.
    --
    new_offset <= '0' when offset_val = offset_reg else '1';

    process(clk, rst)
    begin
        if (rst = '1') then
            nsp <= '1';
        elsif (clk'event and clk = '1') then
            if (ce = '1') then
                if (trs_detected = '1' and trs_error = '0') then
                    nsp <= not frame_en and new_offset;
                end if;
            end if;
        end if;
    end process;

    --
    -- barrel_in: barrel shifter input register
    --
    -- This register implements a pipeline delay stage so that the
    -- barrel shifter's data input matches the delay on the offset
    -- input caused by the offset_reg.
    --
    process(clk, rst)
    begin
        if (rst = '1') then
            barrel_in <= (others => '0');
        elsif (clk'event and clk = '1') then
            if (ce = '1') then
                barrel_in <= in2_reg(8 downto 0) & in3_reg;
            end if;
        end if;
    end process;

    barrel_in_s <= TO_STDLOGICVECTOR(barrel_in(17 downto 0));

    --
    -- barrel shifter
    --
    -- The barrel shifter extracts a 10-bit field from the 19-bit barrel_in 
    -- vector. The bits extracted depend on the value of the offset_reg. If 
    -- offset_reg is zero, barrel_in[9:0] are output by the barrel shifter. At 
    -- the other extreme, an offset_reg value of 9 causes the barrel shifter to 
    -- output barrel_in[18:9].
    --
    -- In this implementation, the nine LS bits of the barrel shifter are 
    -- implemented with a Virtex II embedded 18x18 multiplier. The LS 18-bits of
    -- barrel_in are connected to the A input of the multiplier. The B input of 
    -- the multiplier is connected to a shiftcode value derived from the value 
    -- in the offset_reg. The shiftcode is a 10-bit unary bit code. Since only 
    -- one bit of shiftcode is set at a time, the shiftcode essentially 
    -- multiplies the barrel_in value by a power of two from 1 (when the 
    -- offset_reg value is 9) to 512 (when the offset_reg value is 0).
    --
    -- The barrel shifter's outputs are taken from bits [17:9] of the 
    -- multiplier's product output port. When the offset_reg contains a value of
    -- 9, the barrel_in vector is multiplied by a shiftcode of 1, causing bits 
    -- [17:9] of barrel_in to pass directly to the multiplier's output bits 
    -- [17:9]. At the other extreme, when the offset_reg contains a value of 0, 
    -- the barrel_in vector is multiplied by a shiftcode of 512, causing bits 
    -- [8:0] of barrel_in to be shifted up so they appear on the multiplier's 
    -- output bits [17:9].
    --
    -- The MS bit of the barrel shifter is produced with a 10:1 MUX using a case
    -- statement.
    --

    --
    -- Instantiate the 18x18 multiplier
    --
    sc <= std_logic_vector("00000000" & offset_reg);

    BS : MULT18X18
        port map (
            A => barrel_in_s,
            B => sc,
            P => barrel_out
        );

    --
    -- Generate a 10:1 MUX for the MSB of the barrel shifter
    --
    barrel_msb <= (barrel_in( 9) and offset_reg(9)) or
                  (barrel_in(10) and offset_reg(8)) or
                  (barrel_in(11) and offset_reg(7)) or
                  (barrel_in(12) and offset_reg(6)) or
                  (barrel_in(13) and offset_reg(5)) or
                  (barrel_in(14) and offset_reg(4)) or
                  (barrel_in(15) and offset_reg(3)) or
                  (barrel_in(16) and offset_reg(2)) or
                  (barrel_in(17) and offset_reg(1)) or
                  (barrel_in(18) and offset_reg(0));

    --
    -- The output of the module is the barrel shifter output
    --
--    q <= barrel_msb & std_ulogic_vector(8 downto 0)(barrel_out(17 downto 9));
    barrel_out_u <= TO_STDULOGICVECTOR(barrel_out(17 downto 9));
    q <= barrel_msb & barrel_out_u;

    --
    -- trs: trs output generation logic
    --
    -- The trs_out register is a 4-bit shift register which shifts everytime
    -- the bit_cntr(0) bit is asserted. The trs output signal is the OR of
    -- the four bits in this register so it becomes asserted when the first
    -- character of the TRS symbol is output and remains asserted for the
    -- following three characters of the TRS symbol.
    --
    process(clk, rst)
    begin
        if (rst = '1') then
            trs_out <= (others => '0');
        elsif (clk'event and clk = '1') then
            if (ce = '1') then
                trs_out <= trs_detected & trs_out(3 downto 1);
            end if;
        end if;
    end process;

    trs <= trs_out(3) or trs_out(2) or trs_out(1) or trs_out(0);

end synth;