-------------------------------------------------------------------
-- par_scrambler.vhd
--
-- SDI Parallel Scrambler module
--
--
--
--                  Author: John F. Snow
--                  Staff Applications Engineer
--
--                  Video Applications
--                  Advanced Products Group
--                  Xilinx, Inc.
--
--                  Copyright (c) 2001 Xilinx, Inc.
--                  All rights reserved
--
--                  Date:   Oct. 2, 2001
--
--                  RESTRICTED RIGHTS LEGEND
--
--      This software has not been published by the author, and 
--      has been disclosed to others for the purpose of enhancing 
--      and promoting design productivity in Xilinx products.
--
--      Therefore use, duplication or disclosure, now and in the 
--      future should give consideration to the productivity 
--      enhancements afforded the user of this code by the author's 
--      efforts.  Thank you for using our products !
--
-- Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
-- Revision:
--          Oct. 2, 2001     Creation
--
--
-- Other modules instanced in this design:
--
--          None.
-------------------------------------------------------------------
-- 
-- Description of module:
-- 
-- SMPTE 259M-1997 serial digital interface (SDI) is a standard for transmitting
-- digital video over a serial link.
-- 
-- SDI specifies that the serial bit stream shall be encoded in two ways. First,
-- a generator polynomial of x^9 + x^4 + 1 is used to generate a scrambled NRZ 
-- bit stream. Next, a generator polynomial of x + 1 is used to produce the 
-- final polarity free NRZI bit stream which is transmitted over the physical 
-- layer.
-- 
-- The scrambler module described in this file sits at the transmitting end of 
-- the SDI link and divides the video bit stream by the two generator 
-- polynomials.
-- 
-- The nrzi input, when high, enables the NRZ-to-NRZI conversion. If this input 
-- is low, this conversion is bypassed.
-- 
-- The scram input, when high, enables the SDI scrambler. If this input is low, 
-- the scrambler is bypassed.
-- 
-- This module is a parallel implementation of the scrambler. It accepts one 
-- 10-bit data word in every clock cycle and generates one 10-bit word out every
-- clock cycle. The ld signal causes all registers in the module to load and 
-- serves as a clock enable for the module. 
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity par_scrambler is
    port (
        clk:        in  std_ulogic;                     -- clock input
        rst:        in  std_ulogic;                     -- async reset input
        ld:         in  std_ulogic;                     -- load enable input
        nrzi:       in  std_ulogic;                     -- enables NRZ-to-NRZI conversion when high
        scram:      in  std_ulogic;                     -- enables scrambler when high
        d:          in  std_ulogic_vector(9 downto 0);  -- input data bus
        q:          out std_ulogic_vector(9 downto 0)   -- encoded data out
    );
end par_scrambler;

architecture synth of par_scrambler is

-- Internal signal
signal scram_reg :      std_ulogic_vector(9 downto 0);  -- pipeline delay reg
signal out_reg :        std_ulogic_vector(9 downto 0);  -- output register
signal scram_in :       std_ulogic_vector(13 downto 0); -- input to scrambler
signal scram_temp :     std_ulogic_vector(9 downto 0);  -- intermediate output of scrambler
signal scram_out :      std_ulogic_vector(9 downto 0);  -- output of scrambler
signal nrzi_out :       std_ulogic_vector(9 downto 0);  -- output of NRZ-to-NRZI
signal nrzi_temp :      std_ulogic_vector(9 downto 0);  -- intermediate output of NRZ-to-NRZI

begin
    
    --
    -- SDI Scrambler
    --
    -- This block of logic implements the SDI scrambler algorithm. The scrambler
    -- uses the 10 incoming bits from the input port and a 14-bit vector called 
    -- scram_in. scram_in is made up of 9 bits that were scrambled in the 
    -- previous clock cycle and are stored in the 9 MSBs of scram_reg and the 5 
    -- LS scrambled bits that have been generated during the current clock 
    -- cycle. The results of the scrambler are assigned to scram_temp.
    --
    -- A MUX will output either the value of scram_temp or the d input word
    -- depending on the scram enable input. The output of the MUX is stored in 
    -- the scram_reg.
    --
    scram_in <= scram_temp(4 downto 0) & scram_reg(9 downto 1);

    process(d, scram_in)
    begin
        for i in 0 to 9 loop
            scram_temp(i) <= d(i) xor scram_in(i) xor scram_in(i + 4);
        end loop;   
    end process;

    with scram select
        scram_out <= scram_temp when '1',
                     d when others;

    process(clk, rst)
    begin
        if (rst = '1') then
            scram_reg <= (others => '0');
        elsif (clk'event and clk = '1') then
            if (ld = '1') then
                scram_reg <= scram_out;
            end if;
        end if;
    end process;

    --
    -- NRZ-to-NRZI converter
    --
    -- This block of logic implements the NRZ-to-NRZI conversion. It operates on
    -- the 10 bits coming from the scram_reg and the MSB stored in the out_reg. 
    -- A MUX bypasses the conversion process if the nrzi input is low.
    --
    process(scram_reg, out_reg, nrzi_out)
    begin
        nrzi_temp(0) <= out_reg(9) xor scram_reg(0);
        for j in 1 to 9 loop
            nrzi_temp(j) <= nrzi_out(j - 1) xor scram_reg(j);
        end loop;
    end process;

    with nrzi select
        nrzi_out <= nrzi_temp when '1',
                    scram_reg when others;

    --
    -- out_reg: Output register
    --
    process(clk, rst)
    begin
        if (rst = '1') then
            out_reg <= (others => '0');
        elsif (clk'event and clk = '1') then
            if (ld = '1') then
                out_reg <= nrzi_out;
            end if;
        end if;
    end process;

    q <= out_reg;

end synth;
