-------------------------------------------------------------------------------
-- sdi_tx_ddr.v
--
-- SDI transmitter module
--
--
--
--                  Author: John F. Snow
--                  Staff Applications Engineer
--
--                  Video Applications
--                  Advanced Products Group
--                  Xilinx, Inc.
--
--                  Copyright (c) 2003 Xilinx, Inc.
--                  All rights reserved
--
--                  Date:   Oct. 28, 2003
--
--                  RESTRICTED RIGHTS LEGEND
--
--      This software has not been published by the author, and 
--      has been disclosed to others for the purpose of enhancing 
--      and promoting design productivity in Xilinx products.
--
--      Therefore use, duplication or disclosure, now and in the 
--      future should give consideration to the productivity 
--      enhancements afforded the user of this code by the author's 
--      efforts.  Thank you for using our products !
--
-- Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
-- Revision:
--          Oct. 28, 2003     Release 1.0
--
--
-- Other modules instanced in this design:
--          par_scrambler (from XAPP298)
--
-------------------------------------------------------------------------------
--
-- Module description:
-- 
-- This module instantiates the SDI scrambler, par_scrambler, and implements the
-- serializer. The scrambler runs at the word rate, encoding 10-bits every cycle
-- of the 27MHz word clock. The serializer loads once every 5 cycles of the
-- 135MHz clock from the output of the scrambler module. The serializer produces
-- two bits every cycle of the 135MHz clock to feed a FDDRRSE primitive.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library unisim ;                
use unisim.vcomponents.all ;

entity sdi_tx_ddr is
    port (
        sclk :          in  std_ulogic;                     -- half bit-rate clock
        pclk :          in  std_ulogic;                     -- word rate clock
        rst :           in  std_ulogic;                     -- reset input
        vid_in :        in  std_ulogic_vector(9 downto 0);  -- parallel unscrambled data in
        q :             out std_ulogic_vector(1 downto 0)   -- encoded serial data output bits
    );
end sdi_tx_ddr;

architecture synth of sdi_tx_ddr is

component par_scrambler
    port(
        clk         : in  std_ulogic;
        rst         : in  std_ulogic;
        ld          : in  std_ulogic;
        nrzi        : in  std_ulogic;
        scram       : in  std_ulogic;
        d           : in  std_ulogic_vector(9 downto 0);
        q           : out std_ulogic_vector(9 downto 0)
    );
end component;  

-- Internal signal definitions
signal VCC :            std_ulogic;
signal GND :            std_ulogic;
signal scram_vid :      std_ulogic_vector(9 downto 0);
signal sync_reg :       std_ulogic_vector(9 downto 0);
signal par2ser :        std_ulogic_vector(9 downto 0);
signal par2ser_ld :     std_ulogic;
signal ld_gen :         std_ulogic_vector(2 downto 0);

begin

    VCC <= '1';
    GND <= '0';

    ------------------------------------------------------------------------------
    -- SMPTE 259M encoder module
    --
    -- This module encodes the video per the SMPTE 259M standard.
    --
    SCRAM : par_scrambler 
        port map (
            clk         => pclk,
            rst         => rst,
            ld          => VCC,     -- load every clock cycle
            nrzi        => VCC,     -- enable the NRZ-to-NRZI converter
            scram       => VCC,     -- enable the SMPTE scrambler
            d           => vid_in,
            q           => scram_vid);
         
    ------------------------------------------------------------------------------
    -- Synchronization and serialization
    --

    --
    -- Sync register
    --
    -- This register holds the output of the scrambler module. It clocks 
    -- synchronously with the word-rate clock, but its output feeds the serializer
    -- register that clocks synchronously with the sclk.
    --
    process(pclk, rst)
    begin
        if rst = '1' then
            sync_reg <= (others => '0');
        elsif pclk'event and pclk = '1' then
            sync_reg <= scram_vid;
        end if;
    end process;

    --
    -- Load enable generator
    --
    -- This logic generates the parallel load enable input to the serializer
    -- register. Three cascaded flip-flops, clocked by the sclk, delay
    -- the leading edge of the word-rate clock. The first two flip-flops act as
    -- a dual-rank synchronizer to eliminate meta-stable conditions. The load enable
    -- to the serializer is asserted for the one clock cycle when the second
    -- flip-flop is asserted but the third is not.
    --
    process(sclk, rst)
    begin
        if rst = '1' then
            ld_gen <= (others => '0');
        elsif sclk'event and sclk = '1' then
            ld_gen <= (ld_gen(1 downto 0) & pclk);
        end if;
    end process;

    par2ser_ld <= ld_gen(1) and not ld_gen(2);

    --
    -- Serializer register
    --
    -- This register runs at the sclk rate. It loads in parallel when the load
    -- enable is asserted, otherwise it shifts MSB->LSB, two bits every sclk.
    --
    process(sclk, rst)
    begin
        if rst = '1' then
            par2ser <= (others => '0');
        elsif sclk'event and sclk = '1' then
            if par2ser_ld = '1' then
                par2ser <= sync_reg;
            else
                par2ser <= ("00" & par2ser(9 downto 2));
            end if;
        end if;
    end process;

    q(0) <= par2ser(0);
    q(1) <= par2ser(1);

end synth;
