--------------------------------------------------------------------------------
-- sdi_tx_top.vhd
--
-- SMPTE 259M SDI top level Tx & Rx with pattern generator
--
--
--
--                  Author: John F. Snow
--                  Staff Applications Engineer
--
--                  Video Applications
--                  Advanced Products Group
--                  Xilinx, Inc.
--
--                  Copyright (c) 2003 Xilinx, Inc.
--                  All rights reserved
--
--                  Date:   Oct. 28, 2003
--
--                  RESTRICTED RIGHTS LEGEND
--
--      This software has not been published by the author, and 
--      has been disclosed to others for the purpose of enhancing 
--      and promoting design productivity in Xilinx products.
--
--      Therefore use, duplication or disclosure, now and in the 
--      future should give consideration to the productivity 
--      enhancements afforded the user of this code by the author's 
--      efforts.  Thank you for using our products !
--
-- Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
-- Revision:
--          Oct 28, 2003        Release 1.0
--                                                          
-- Other modules instanced in this design:
--          sdi_tx_ddr
--
--------------------------------------------------------------------------------
-- 
-- Description of module:
-- 
-- This is the top level module for an SMPTE 259M SDI transmitter. This module 
-- instances all the IOBs and the DCM required to implement the SDI transmitter.
-- The actual SDI endcoder and serializer are implemented in the sdi_tx_ddr 
-- module.
-- 
-- This SDI transmitter uses DDR techniques to generate the 270Mbps bitstream. 
-- The DCM takes in a 27MHz word-rate video clock and generates two phases of a 
-- 135MHz clock. 135MHz is one-half the 270Mbps bit rate of SDI. The two 135MHz 
-- clock phases are 180 degrees apart in phase and are used to clock the DDR 
-- flip-flop primitive FDDRRSE. The sdi_tx_ddr module produces two bits per 
-- cycle of the 135MHz clock. The FDDRRSE primitive serializes these two bits to
-- produce a 270Mbps bitstream.
-- 
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library unisim ;                
use unisim.vcomponents.all ;

entity sdi_tx_top is
    port (
        clk27M:         in  std_ulogic;                     -- 27MHz reference clock input
        clkfb_in:       in  std_ulogic;                     -- DCM feedback clock input
        vid_in:         in  std_ulogic_vector(9 downto 0);  -- input video
        push_button1:   in  std_ulogic;                     -- resets all logic (GSR)
        sdi_tx_dout_p:  out std_ulogic;                     -- SDI_Tx differential data output
        sdi_tx_dout_n:  out std_ulogic;
        clk_out:        out std_ulogic                      -- 27MHz clock output for DCM feedback
    );
end sdi_tx_top;

architecture synth of sdi_tx_top is

component sdi_tx_ddr
    port(
        sclk :                  in  std_ulogic;
        pclk :                  in  std_ulogic;
        rst :                   in  std_ulogic;
        vid_in :                in  std_ulogic_vector(9 downto 0);
        q :                     out std_ulogic_vector(1 downto 0)
    );
end component;

-- Internal signal
signal clk27M_int:      std_ulogic;                     -- clk27M input clock after IBUFG
signal tx_sclk_p:       std_ulogic;                     -- CLKFX output of DCM after BUFG
signal tx_sclk_n:       std_ulogic;                     -- CLKFX180 output of DCM after BUFG
signal tx_pclk:         std_ulogic;                     -- CLK0 output of DCM after BUFG
signal tx_serout:       std_ulogic_vector(1 downto 0);  -- serial output of SDI transmitter
signal pb1:             std_ulogic;                     -- buffered push_button1 (reset)
signal tx_dcm_locked:   std_ulogic;                     -- LOCKED output of DCM
signal tx_ddr_out:      std_ulogic;                     -- serial output of the FDDRRSE primitive
signal dcm_clk_fb:      std_ulogic;                     -- feedback clock for the DCM after going through IOB
signal clk0_buf:        std_ulogic;                     -- CLK0 output of DCM
signal clkfx_buf:       std_ulogic;                     -- CLKFX output of DCM
signal clkfx180_buf:    std_ulogic;                     -- CLKFX180 output of DCM
signal video_in:        std_ulogic_vector(9 downto 0);  -- parallel video input after IOBs
signal VCC:             std_ulogic;
signal GND:             std_ulogic;

attribute CLK_FEEDBACK :        string;
attribute CLKDV_DIVIDE :        real;
attribute CLKFX_DIVIDE :        integer;
attribute CLKFX_MULTIPLY :      integer;
attribute CLKIN_DIVIDE_BY_2 :   string;
attribute CLKIN_PERIOD :        string;
attribute CLKOUT_PHASE_SHIFT :  string;
attribute DESKEW_ADJUST :       string;
attribute DFS_FREQUENCY_MODE :  string;
attribute DLL_FREQUENCY_MODE :  string;
attribute DSS_MODE :            string;
attribute DUTY_CYCLE_CORRECTION:string;
attribute PHASE_SHIFT :         real;
attribute STARTUP_WAIT :        string;

attribute CLK_FEEDBACK          of DCM1 : label is "1X";
attribute CLKDV_DIVIDE          of DCM1 : label is 2.0;
attribute CLKFX_DIVIDE          of DCM1 : label is 1;
attribute CLKFX_MULTIPLY        of DCM1 : label is 5;
attribute CLKIN_DIVIDE_BY_2     of DCM1 : label is "FALSE";
attribute CLKIN_PERIOD          of DCM1 : label is "27.778";
attribute CLKOUT_PHASE_SHIFT    of DCM1 : label is "NONE";
attribute DESKEW_ADJUST         of DCM1 : label is "SYSTEM_SYNCHRONOUS";
attribute DFS_FREQUENCY_MODE    of DCM1 : label is "LOW";
attribute DLL_FREQUENCY_MODE    of DCM1 : label is "LOW";
attribute DSS_MODE              of DCM1 : label is "NONE";
attribute DUTY_CYCLE_CORRECTION of DCM1 : label is "TRUE";
attribute PHASE_SHIFT           of DCM1 : label is 0.0;
attribute STARTUP_WAIT          of DCM1 : label is "FALSE";


begin
    
-------------------------------------------------------------------------------
-- I/O Buffers
--

    VCC <= '1';
    GND <= '0';

    --
    -- Instantiate the startup block
    --
    -- The pushbutton input is used to issue a GSR reset to the FPGA.
    --
    STARTV2 : STARTUP_VIRTEX2 
        port map (
            CLK         => GND,
            GSR         => not pb1,
            GTS         => GND);
                                                  
    --
    -- Push button reset switch
    --
    PB1_IN : IBUF_LVCMOS25 
        port map (
            I           => push_button1, 
            O           => pb1);

    --
    -- 27MHz input clock buffer
    --
    CLK27 : IBUFG_LVCMOS25 
        port map (
            O           => clk27M_int,
            I           => clk27M);

    --
    -- This DCM produces a de-skewed 27MHz clock output (CLK0) and two phases of
    -- a 135MHz clock that are 180 degress out of phase.
    --
    DCM1 : DCM
        -- synthesis translate_off
        generic map (
            CLK_FEEDBACK            => "1X";
            CLKDV_DIVIDE            => 2.0;
            CLKFX_DIVIDE            => 1;
            CLKFX_MULTIPLY          => 5;
            CLKIN_DIVIDE_BY_2       => FALSE;
            CLKOUT_PHASE_SHIFT      => "NONE";
            DESKEW_ADJUST           => "SYSTEM_SYNCHRONOUS";
            DFS_FREQUENCY_MODE      => "LOW";
            DLL_FREQUENCY_MODE      => "LOW";
            DSS_MODE                => "NONE";
            DUTY_CYCLE_CORRECTION   => "TRUE";
            PHASE_SHIFT             => 0;
            STARTUP_WAIT            => FALSE)
        -- synthesis translate_on
        port map (
            CLKIN       => clk27M_int,
            CLKFB       => dcm_clk_fb,
            RST         => GND,
            PSEN        => GND,
            PSINCDEC    => GND,
            PSCLK       => GND,
            DSSEN       => GND,
            CLK0        => clk0_buf,
            CLK90       => open,
            CLK180      => open,
            CLK270      => open,
            CLKDV       => open,
            CLK2X       => open,
            CLK2X180    => open,
            CLKFX       => clkfx_buf,
            CLKFX180    => clkfx180_buf,
            STATUS      => open,
            LOCKED      => tx_dcm_locked,
            PSDONE      => open);
    --
    -- This is the input buffer for the feedback clock for the DCM.
    --
    CLKFB_IBUFG_INST : IBUFG_LVCMOS25 
        port map (
            I           => clkfb_in,
            O           => dcm_clk_fb);

    --
    -- This is the outpout buffer for the CLK0 output of the DCM. Normally this
    -- output would be wired to the clkfb_in pin of the FPGA with suitable PCB
    -- routing to match the routing delay of the video inputs.
    --
    CLKOUT : OBUF_LVCMOS2 
        port map (
            I           => tx_pclk,
            O           => clk_out);

    -- 
    -- This is the BUFG for the CLK0 output of the DCM (27MHz).
    --
    CLK0_BUFG_INST : BUFG 
        port map (
            I           => clk0_buf,
            O           => tx_pclk);

    --
    -- This is the BUFG for the CLKFX output of the DCM (135MHz).
    --
    CLKFX_BUFG_INST : BUFG 
        port map (
            I           => clkfx_buf,
            O           => tx_sclk_p);

    --
    -- This is the BUFG for the CLKFX180 output of the DCM (135MHz).
    --
    CLKFX180_BUFG_INST : BUFG 
        port map (
            I           => clkfx180_buf,
            O           => tx_sclk_n);


    --
    -- SDI transmitter
    --
    -- The sdi_tx_ddr module contains the SDI encoder and the serializer. The
    -- serializer must produce two bit for every cycle of the 135MHz clock in
    -- order to interface with the DDR primitive.
    --  
    SDITX : sdi_tx_ddr 
        port map (
            sclk        => tx_sclk_p,
            pclk        => tx_pclk,
            rst         => GND,
            vid_in      => video_in,
            q           => tx_serout);

    --
    -- SDI-Tx Port transmit data buffer
    --
    -- This is the OBUF for for the SDI output bitstream. It uses an LVDS buffer
    -- to minimize jitter. This output is normally connected to a SDI compatible
    -- cable driver chip with LVDS inputs such as the National Semiconductor
    -- CLC001.
    --
    SDI_TXD : OBUFDS_LVDS_25 
        port map (
            I           => tx_ddr_out, 
            O           => sdi_tx_dout_p, 
            OB          => sdi_tx_dout_n);

    --
    -- DDR primitive
    -- 
    -- The primitive contains two flip-flops and a MUX that convert the two bits
    -- from the sdi_tx_ddr module into a 270Mbps bitstream.
    --
    DDR : FDDRRSE 
        port map (
            D0          => tx_serout(1),
            D1          => tx_serout(0),
            C0          => tx_sclk_p,
            C1          => tx_sclk_n,
            CE          => VCC,
            R           => GND,
            S           => GND,
            Q           => tx_ddr_out);
                

    --
    -- Input video buffers
    --
    -- These are the input buffers for the parallel video.
    --
    VID0 : IBUF_LVCMOS25 
        port map (
            O           => video_in(0),
            I           => vid_in(0));

    VID1 : IBUF_LVCMOS25 
        port map (
            O           => video_in(1),
            I           => vid_in(1));

    VID2 : IBUF_LVCMOS25 
        port map (
            O           => video_in(2),
            I           => vid_in(2));

    VID3 : IBUF_LVCMOS25 
        port map (
            O           => video_in(3),
            I           => vid_in(3));

    VID4 : IBUF_LVCMOS25 
        port map (
            O           => video_in(4),
            I           => vid_in(4));

    VID5 : IBUF_LVCMOS25 
        port map (
            O           => video_in(5),
            I           => vid_in(5));

    VID6 : IBUF_LVCMOS25 
        port map (
            O           => video_in(6),
            I           => vid_in(6));

    VID7 : IBUF_LVCMOS25 
        port map (
            O           => video_in(7),
            I           => vid_in(7));

    VID8 : IBUF_LVCMOS25 
        port map (
            O           => video_in(8),
            I           => vid_in(8));

    VID9 : IBUF_LVCMOS25 
        port map (
            O           => video_in(9),
            I           => vid_in(9));

end synth;