--------------------------------------------------------------------------------
-- x247_rx_ext_cdr.vhd
--
-- SMPTE 259M SDI Receiver using external CDR chip
--
--
--
--                  Author: John F. Snow
--                  Staff Applications Engineer
--
--                  Video Applications
--                  Advanced Products Group
--                  Xilinx, Inc.
--
--                  Copyright (c) 2003 Xilinx, Inc.
--                  All rights reserved
--
--                  Date:   Oct. 28, 2003
--
--                  RESTRICTED RIGHTS LEGEND
--
--      This software has not been published by the author, and 
--      has been disclosed to others for the purpose of enhancing 
--      and promoting design productivity in Xilinx products.
--
--      Therefore use, duplication or disclosure, now and in the 
--      future should give consideration to the productivity 
--      enhancements afforded the user of this code by the author's 
--      efforts.  Thank you for using our products !
--
-- Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
-- Revision:
--              
--              Oct 28, 2003        Release 1.0
--                                          
-- Other modules instanced in this design:
--      par_descrambler (from XAPP288)
--      par_framer_mult (from XAPP288)
--------------------------------------------------------------------------------
-- 
-- Description of module:
-- 
-- This module is a SDI receiver that uses an external clock & data recovery 
-- chip or "reclocker". This design has been tested with the National 
-- Semiconductor CLC016 SDI CDR.
-- 
-- The CLC016 produces two differential outputs. One is the recoverd bit-rate
-- clock. The other is the original bitstream that has be synchronized to the
-- recovered clock. The serial clock is used by the FPGA to clock a shift
-- register to deserialize the bitstream and produce 10-bit words. The serial
-- clock is divided by 10 to get a word-rate clock. The word-rate clock drives 
-- the descrambler and framer modules that take the encoded bitstream and 
-- produce decoded and framed video words.
-- 
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library unisim ;                
use unisim.vcomponents.all ;

entity x247_rx_ext_cdr is
    port (
        mr_sd_rxc_p :   in  std_ulogic;                     -- MR-Rx SDI differential clock input from CLC016
        mr_sd_rxc_n :   in  std_ulogic;
        mr_sd_rxd_p :   in  std_ulogic;                     -- MR-Rx SDI differential data input from CLC016
        mr_sd_rxd_n :   in  std_ulogic;
        mr_sd_ser :     in  std_ulogic;                     -- asserted low when CLC016 is locked
        mr_sd_cd :      in  std_ulogic;                     -- carrier detect input from CLC016
        push_button1 :  in  std_ulogic;                     -- resets all logic when low
        video_out :     out std_ulogic_vector(9 downto 0);  -- parallel video output
        trs :           out std_ulogic;                     -- asserted when receiver detects TRS symbols in bitstream
        video_clk :     out std_ulogic                      -- recovered word-rate video clock output
    );
end x247_rx_ext_cdr;

architecture synth of x247_rx_ext_cdr is
    
component par_descrambler
    port(
        clk:            in  std_ulogic;
        rst:            in  std_ulogic;
        ld:             in  std_ulogic;
        d:              in  std_ulogic_vector(9 downto 0);
        q:              out std_ulogic_vector(9 downto 0)
    );
end component;  

component par_framer_mult
    port(
        clk:            in  std_ulogic;
        rst:            in  std_ulogic;
        ce:             in  std_ulogic;
        d:              in  std_ulogic_vector(9 downto 0);
        frame_en:       in  std_ulogic;
        q:              out std_ulogic_vector(9 downto 0);
        trs:            out std_ulogic;
        nsp:            out std_ulogic
    );
end component;

-------------------------------------------------------------------------------
-- Signal definitions
--
signal ringcntr :       std_ulogic_vector(4 downto 0);  -- divide by 10 counter to generate rxrecclk
signal ser2par_reg :    std_ulogic_vector(9 downto 0);  -- deserialization register
signal sync_reg :       std_ulogic_vector(9 downto 0);  -- sync reg between clock domains
signal desc_in_reg :    std_ulogic_vector(9 downto 0);  -- input register to SMPTE descrambler
signal desc_out :       std_ulogic_vector(9 downto 0);  -- output of SMPTE descrambler
signal frame_en :       std_ulogic;                     -- framer enable
signal framer_out :     std_ulogic_vector(9 downto 0);  -- video out of framer
signal framer_nsp :     std_ulogic;                     -- framer new-start-position signal
signal framer_trs :     std_ulogic;                     -- framer TRS output signal
signal vid_reg :        std_ulogic_vector(9 downto 0);  -- reg between framer and edh_processor
signal rx_sclk_int :    std_ulogic;                     -- receiver bit-rate clock after IOB
signal rx_pclk_int :    std_ulogic;                     -- receiver word-rate clock before BUFG
signal rx_sclk :        std_ulogic;                     -- global receiver bit-rate clock
signal rx_pclk :        std_ulogic;                     -- global receiver word-rate clock
signal rx_ser_dat :     std_ulogic;                     -- rx serial data input before input FF
signal rx_sdat :        std_ulogic;                     -- rx serial data input FF
signal clc_ser :        std_ulogic;                     -- indicates locked status of CDR chip
signal clc_cd :         std_ulogic;                     -- carrier detect signal from CDR chip
signal clc_locked :     std_ulogic;                     -- inverted version of clc_ser
signal rxrecclk :       std_ulogic;                     -- input to BUFG that drives rx_pclk
signal rx_video :       std_ulogic_vector(9 downto 0);  -- video from receiver
signal pb1 :            std_ulogic;                     -- buffered push_button1 (reset)
signal sync_ld :        std_ulogic;                     -- loads the sync_reg
signal rst :            std_ulogic;                     -- async reset from pb1
signal VCC :            std_ulogic;
signal GND :            std_ulogic;

attribute init : string;
attribute init of RCNTR : label is "00000";

attribute iobdelay : string;
attribute iobdelay of MRSDDAT : label is "ifd";

attribute iob : string;
attribute iob of RX_SDAT_FF : label is "true";

begin

    VCC <= '1';
    GND <= '0';

    --
    -- Instantiate the startup block
    --
    STARTV2 : STARTUP_VIRTEX2 
        port map (
            CLK         => GND,
            GSR         => not pb1,
            GTS         => GND);

                                                  
    --
    -- Push button reset switch input
    --
    PB1_IN : IBUF_LVCMOS25
        port map (
            O           => pb1,
            I           => push_button1);

    rst <= not pb1;

    --
    -- Parallel video outputs
    --
    VIDOUT_0 : OBUF_LVCMOS25 
        port map (
            O           => video_out(0),
            I           => rx_video(0));

    VIDOUT_1 : OBUF_LVCMOS25 
        port map (
            O           => video_out(1),
            I           => rx_video(1));

    VIDOUT_2 : OBUF_LVCMOS25 
        port map (
            O           => video_out(2),
            I           => rx_video(2));

    VIDOUT_3 : OBUF_LVCMOS25 
        port map (
            O           => video_out(3),
            I           => rx_video(3));

    VIDOUT_4 : OBUF_LVCMOS25 
        port map (
            O           => video_out(4),
            I           => rx_video(4));

    VIDOUT_5 : OBUF_LVCMOS25 
        port map (
            O           => video_out(5),
            I           => rx_video(5));

    VIDOUT_6 : OBUF_LVCMOS25 
        port map (
            O           => video_out(6),
            I           => rx_video(6));

    VIDOUT_7 : OBUF_LVCMOS25 
        port map (
            O           => video_out(7),
            I           => rx_video(7));

    VIDOUT_8 : OBUF_LVCMOS25 
        port map (
            O           => video_out(8),
            I           => rx_video(8));

    VIDOUT_9 : OBUF_LVCMOS25 
        port map (
            O           => video_out(9),
            I           => rx_video(9));


    --
    -- Input buffers for the serial clock and data coming from CDR chip
    --
    -- The IOBDELAY attribute is attached to the serial data input IOB. This
    -- adds the IOBDELAY element to the data path before the IOB flip-flop. This
    -- delay matches the global clock distribution delay that is added to the
    -- serial clock so that setup and hold time are met on the input to the
    -- IOB flip-flop.
    --
    MRSDCLK : IBUFDS_LVDSEXT_25_DCI 
        port map (
            O           => rx_sclk_int,
            I           => mr_sd_rxc_p,
            IB          => mr_sd_rxc_n);

    MRSDDAT : IBUFDS_LVDSEXT_25_DCI 
        port map (
            O           => rx_ser_dat,
            I           => mr_sd_rxd_p,
            IB          => mr_sd_rxd_n);

    --
    -- Input FF for serial bit stream
    --
    -- This FF is located in the IOB to minimize the delay time on the data
    --
    RX_SDAT_FF : process(rx_sclk)
    begin
        if rx_sclk'event and rx_sclk = '1' then
            rx_sdat <= rx_ser_dat;
        end if;
    end process;


    --
    -- Input buffer for the SER signal from CDR chip
    --
    -- The SER output of the CLC016 will be low when the CLC016 is locked to the
    -- incoming bitstream. This can be used as an indicator to internal logic that
    -- the CDR is locked.
    --
    MDSD_SER : IBUF_LVDCI_25 
        port map (
            O           => clc_ser,
            I           => mr_sd_ser);

    clc_locked <= not clc_ser;

    --
    -- Input buffer for the CD signal from the CDR chip
    --
    -- The CD output of the CLC016 will be high when the CLC016 detects an input
    -- signal. It can be used as another status input to the logic in the FPGA.
    --
    MDSD_CD : IBUF_LVDCI_25 
        port map (
            O           => clc_cd,
            I           => mr_sd_cd);

    --
    -- Global clock buffers
    --
    -- The rx_sclk is the globally distributed version of the 270MHz bit-rate 
    -- recovered clock from the CLC016 CDR. The rx_pclk is the global version of
    -- the word-rate clock that is generated by dividing rx_sclk by 10.
    --
    RXSCLK_BUF : BUFG
        port map (
            O           => rx_sclk,
            I           => rx_sclk_int);


    RXPCLK_BUF : BUFG
        port map (
            O           => rx_pclk, 
            I           => rxrecclk);


    -------------------------------------------------------------------------------
    -- Receiver Section
    --

    --
    -- Rx parallel clock generation
    --
    -- This code generates a 27MHz clock from the 270M from the CDR PLL. A 5-bit 
    -- ring counter is clocked by the 270M clock. The ring counter first fills up
    -- with all ones and then with all zeros, producing a 50% duty cycle 27MHz 
    -- clock.
    --
    RCNTR : process(rx_sclk)
    begin
        if rx_sclk'event and rx_sclk = '1' then
            ringcntr <= (ringcntr(3 downto 0) & not ringcntr(4));
        end if;
    end process;

    rxrecclk <= ringcntr(4);


    --
    -- Deserialization and synchronization registers
    --
    -- The ser2par_reg is a shift register that deserializes the data. The output
    -- of the ser2par_reg is loaded into the sync_reg at about the same time that
    -- as the falling edge of the rx_pclk. The data in the sync_reg is synchronized 
    -- to the rx_pclk domain by loading the contents of the sync_reg into the 
    -- desc_in_reg on the rising edge of rx_pclk. This gives us about 5 rx_sclk 
    -- cycles of setup time and hold time for the data going into the desc_in_reg, 
    -- minus clock distribution delays.
    --
    process(rx_sclk)
    begin
        if rx_sclk'event and rx_sclk = '1' then
            ser2par_reg <= (rx_sdat & ser2par_reg(9 downto 1));
        end if;
    end process;

    process(rx_sclk)
    begin
        if rx_sclk'event and rx_sclk = '1' then
            if ringcntr(3) = '1' and ringcntr(2) = '0' then
                sync_ld <= '1';
            else
                sync_ld <= '0';
            end if;
        end if;
    end process;

    process(rx_sclk)
    begin
        if rx_sclk'event and rx_sclk = '1' then
            if sync_ld = '1' then
                sync_reg <= ser2par_reg;
            end if;
        end if;
    end process;

    process(rx_pclk)
    begin
        if rx_pclk'event and rx_pclk = '1' then
            desc_in_reg <= sync_reg;
        end if;
    end process;

    --
    -- par_descrambler
    --
    -- The SDI parallel descrambler decodes the bitstream. This module can be found 
    -- in XAPP288.
    --
    SDI_DESC : par_descrambler 
        port map (
            clk         => rx_pclk,
            rst         => rst,
            ld          => VCC,
            d           => desc_in_reg,
            q           => desc_out);

    --
    -- par_framer_mult
    --
    -- The framer synchronizes to TRS symbols and then frames each 10-bit video word
    -- appropriately. There are two versions of the parallel framer available in the
    -- XAPP288 reference design. One uses a MULT18X18 primitive as a barrel shifter,
    -- the other version implements the barrel shifter in logic. The version that 
    -- uses the MULT18X18 primitive saves some CLB logic when a spare MULT18X18 
    -- is available in the FPGA.
    --
    SDI_FRM : par_framer_mult
        port map (
            clk         => rx_pclk,
            rst         => rst,
            ce          => VCC,
            d           => desc_out,
            frame_en    => frame_en,
            q           => framer_out,
            trs         => framer_trs,
            nsp         => framer_nsp);

    frame_en <= framer_nsp; -- if using the EDH processor module, assign frame_en
                            -- to framer_nsp or ep_sync_switch where ep_sync_switch
                            -- is a signal from the EDH processor that indicates
                            -- when the current video is in the synchronous
                            -- switching interval

    --
    -- vid_reg
    --
    -- Holds the parallel data out of framer
    --
    process(rx_pclk)
    begin
        if rx_pclk'event and rx_pclk = '1' then
            vid_reg <= framer_out;
        end if;
    end process;

    rx_video <= vid_reg;

    --
    -- Output buffers for the video_clk and trs outputs
    --
    VIDCLK_OUT : OBUF_LVCMOS25 
        port map (
            I       => rx_pclk,
            O       => video_clk);

    TRS_OUT : OBUF_LVCMOS25 
        port map (
            I       => framer_trs,
            O       => trs);

end synth;
