--------------------------------------------------------------------------------
-- x247_rx_int_cdr.vhd
--
-- SMPTE 259M SDI receiver using XAPP250 for CDR
--
--
--
--                  Author: John F. Snow
--                  Staff Applications Engineer
--
--                  Video Applications
--                  Advanced Products Group
--                  Xilinx, Inc.
--
--                  Copyright (c) 2003 Xilinx, Inc.
--                  All rights reserved
--
--                  Date:   Oct 28, 2003
--
--                  RESTRICTED RIGHTS LEGEND
--
--      This software has not been published by the author, and 
--      has been disclosed to others for the purpose of enhancing 
--      and promoting design productivity in Xilinx products.
--
--      Therefore use, duplication or disclosure, now and in the 
--      future should give consideration to the productivity 
--      enhancements afforded the user of this code by the author's 
--      efforts.  Thank you for using our products !
--
-- Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
--              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
--              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
--
-- Revision:
--      Oct 28, 2003        Release 1.0
--                                                  
-- Other modules instanced in this design:
--      xapp250_cdr (XAPP250 CDR unit -- note VHDL version not available)
--      par_descrambler (from XAPP288)
--      par_framer (from XAPP288)
--
--------------------------------------------------------------------------------
-- 
-- Module description:
-- 
-- This is the top level module for the XAPP247 SDI receiver using internal CDR. 
-- The CDR unit is based on XAPP250 and uses an inexpensive external 270MHz VCO
-- and loop filter combined with a phase detector built in the FPGA to implement
-- a PLL that locks to the frequency of the SDI bitstream. The CDR files from
-- XAPP250 are included in the reference design archive, but newer versions may
-- be available by downloading the XAPP250 reference designs.
-- 
-- After recovering the clock and data, the recovered data is deserialized, 
-- decoded, and framed. The SDI decoder and framer modules are found in XAPP288.
--------------------------------------------------------------------------------
 
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library unisim ;                
use unisim.vcomponents.all ;

entity x247_rx_int_cdr is
    port (
        clk54M_p:       in  std_ulogic;                     -- 54MHz clock input
        clk54M_n:       in  std_ulogic;
        vco270M_p:      in  std_ulogic;                     -- 270MHz VCO input
        vco270M_n:      in  std_ulogic;
        sd_rxd_p:       in  std_ulogic;                     -- SDI serial bit stream input
        sd_rxd_n:       in  std_ulogic;
        sd_rx_cd:       in  std_ulogic;                     -- carrier detect from cable equalizer
        push_button1:   in  std_ulogic;                     -- push button reset
        sdi_vco_up:     out std_ulogic;                     -- faster output to loop filter
        sdi_vco_down:   out std_ulogic;                     -- slower output to loop filter
        video_out:      out std_ulogic_vector(9 downto 0);  -- parallel video output
        video_clk:      out std_ulogic;                     -- output 27MHz video clock
        trs:            out std_ulogic                      -- TRS detected output
    );
end x247_rx_int_cdr;

architecture synth of x247_rx_int_cdr is

component xapp250_cdr
    port(
        vco:            in  std_ulogic;
        refclk:         in  std_ulogic;
        reset:          in  std_ulogic;
        data:           in  std_ulogic;
        force_resync:   in  std_ulogic;
        dataout:        out std_ulogic_vector(1 downto 0);
        vcoisfast:      out std_ulogic;
        vcoisslow:      out std_ulogic;
        mkvcofaster:    out std_ulogic;
        mkvcoslower:    out std_ulogic;
        vco_tc:         out std_ulogic;
        refclk_tc:      out std_ulogic;
        phasedet:       out std_ulogic_vector(15 downto 0);
        clklags:        out std_ulogic_vector(3 downto 0);
        clkleads:       out std_ulogic_vector(3 downto 0);
        clkisfastc2d:   out std_ulogic;
        clkisslowc2d:   out std_ulogic  
    );
end component;

component par_descrambler
    port(
        clk:            in  std_ulogic;
        rst:            in  std_ulogic;
        ld:             in  std_ulogic;
        d:              in  std_ulogic_vector(9 downto 0);
        q:              out std_ulogic_vector(9 downto 0)
    );
end component;  

component par_framer_mult
    port(
        clk:            in  std_ulogic;
        rst:            in  std_ulogic;
        ce:             in  std_ulogic;
        d:              in  std_ulogic_vector(9 downto 0);
        frame_en:       in  std_ulogic;
        q:              out std_ulogic_vector(9 downto 0);
        trs:            out std_ulogic;
        nsp:            out std_ulogic
    );
end component;

-------------------------------------------------------------------------------
-- Signal definitions
--
signal clk54M_ref:      std_ulogic;                     -- clk5M after IOB used as refclk to MGT
signal vco270M_int:     std_ulogic;                     -- 270MHz VCO after after IOB
signal rx_sclk:         std_ulogic;                     -- receiver bit-rate clock after BUFG
signal rx_pclk:         std_ulogic;                     -- receiver word-rate clock after BUFG
signal rx_ser_dat:      std_ulogic;                     -- rx serial data input before input FF
signal rxrecclk:        std_ulogic;                     -- rx_sclk divided by 10
signal rx_video:        std_ulogic_vector(9 downto 0);  -- video from receiver
signal ringcntr:        std_ulogic_vector(4 downto 0);  -- divide by 10 counter to generate rxrecclk
signal ser2par_reg:     std_ulogic_vector(9 downto 0);  -- deserialization register
signal sync_reg:        std_ulogic_vector(9 downto 0);  -- sync reg between clock domains
signal desc_in_reg:     std_ulogic_vector(9 downto 0);  -- input register to SMPTE descrambler
signal desc_out:        std_ulogic_vector(9 downto 0);  -- output of SMPTE descrambler
signal frame_en:        std_ulogic;                     -- framer enable
signal framer_out:      std_ulogic_vector(9 downto 0);  -- video out of framer
signal framer_nsp:      std_ulogic;                     -- framer new-start-position signal
signal framer_trs:      std_ulogic;                     -- TRS output from framer
signal vid_reg:         std_ulogic_vector(9 downto 0);  -- output register
signal pb1:             std_ulogic;                     -- buffered push_button1 (reset)
signal eq_cd:           std_ulogic;                     -- sdi_rx_cd input after IOB
signal dataout:         std_ulogic_vector(1 downto 0);  -- data from data recovery unit
signal make_vco_faster: std_ulogic;                     -- output of phase detector to VCO loop filter
signal make_vco_slower: std_ulogic;                     -- output of phase detector to VCO loop filter
signal sync_ld:         std_ulogic;                     -- loads the sync_reg
signal rst :            std_ulogic;                     -- inverted push_button1 input
signal VCC:             std_ulogic;
signal GND:             std_ulogic;

attribute init : string;
attribute init of RCNTR : label is "00000";

begin

    VCC <= '1';
    GND <= '0';

    --
    -- Instantiate the startup block
    --
    STARTV2 : STARTUP_VIRTEX2 
        port map (
            CLK         => GND,
            GSR         => not pb1,
            GTS         => GND);

                                                  
    --
    -- Push button reset switch input
    --
    PB1_IN : IBUF_LVCMOS25
        port map (
            O           => pb1,
            I           => push_button1);

    rst <= not pb1;

    --
    -- Parallel video outputs
    --
    VIDOUT_0 : OBUF_LVCMOS25 
        port map (
            O           => video_out(0),
            I           => rx_video(0));

    VIDOUT_1 : OBUF_LVCMOS25 
        port map (
            O           => video_out(1),
            I           => rx_video(1));

    VIDOUT_2 : OBUF_LVCMOS25 
        port map (
            O           => video_out(2),
            I           => rx_video(2));

    VIDOUT_3 : OBUF_LVCMOS25 
        port map (
            O           => video_out(3),
            I           => rx_video(3));

    VIDOUT_4 : OBUF_LVCMOS25 
        port map (
            O           => video_out(4),
            I           => rx_video(4));

    VIDOUT_5 : OBUF_LVCMOS25 
        port map (
            O           => video_out(5),
            I           => rx_video(5));

    VIDOUT_6 : OBUF_LVCMOS25 
        port map (
            O           => video_out(6),
            I           => rx_video(6));

    VIDOUT_7 : OBUF_LVCMOS25 
        port map (
            O           => video_out(7),
            I           => rx_video(7));

    VIDOUT_8 : OBUF_LVCMOS25 
        port map (
            O           => video_out(8),
            I           => rx_video(8));

    VIDOUT_9 : OBUF_LVCMOS25 
        port map (
            O           => video_out(9),
            I           => rx_video(9));


    --
    -- 54MHz input clock buffer
    --
    -- The 54Mhz clock is used as a reference clock to the phase detector in the
    -- CDR unit. The phase detector will lock to this clock when not locked to the
    -- SDI bitstream. There is nothing special about using 54MHz. This just happens
    -- to be a clock value that was used on the evaluation board for this design.
    -- The phase detector divides this clock and the VCO output both down to a 
    -- common value of 1 MHz for comparison reasons. By changing the reference
    -- clock divider in the phase detector, difference reference clock frequencies
    -- can be used.
    --
    CLK54 : IBUFGDS_LVDSEXT_25_DCI 
        port map (
            O           => clk54M_ref,
            I           => clk54M_p,
            IB          => clk54M_n);

    --
    -- 270MHz VCO clock input
    --
    -- This is the output of the external 270MHz VCO used as part of the CDR PLL.
    --
    VCO270 : IBUFGDS_LVDSEXT_25_DCI 
        port map (
            O           => vco270M_int,
            I           => vco270M_p,
            IB          => vco270M_n);

    --
    -- Input buffers for SDI bitstream
    --
    SDI_IN : IBUFDS_LVDSEXT_25_DCI 
        port map (
            O           => rx_ser_dat,
            I           => sd_rxd_p,
            IB          => sd_rxd_n);

    --
    -- Input buffer for the carrier detect signal from CLC014 equalizer
    --
    -- In this design, the carrier detect output of the cable equalizer is used
    -- to force the CDR unit to lock to the reference clock rather than the 
    -- bitstream whenever the equalizer detects the absence of an SDI bitstream.
    -- This mechanism is currently used to resync the CDR unit in the absence of
    -- a bitstream because the phase detector currently lacks a lock detection
    -- circuit.
    --
    CLC_CD : IBUF_LVCMOS25 
        port map (
            O           => eq_cd,
            I           => sd_rx_cd);

    --
    -- Global clock buffers
    --
    RXSCLK_BUF : BUFG 
        port map (
            O           => rx_sclk,
            I           => vco270M_int);

    RXPCLK_BUF : BUFG 
        port map (
            O           => rx_pclk,
            I           => rxrecclk);



    -------------------------------------------------------------------------------
    -- Receiver Section
    --

    --
    -- CDR
    --
    -- This module implements the phase detector and data recovery unit for the
    -- SDI receiver. It uses the 54MHz clock as a reference clock. This is a very
    -- slightly modified version of the XAPP250 CDR unit that adds a force_resync
    -- input. This input is connected to the cable equalizer carrier detect
    -- signal to force the PLL to lock to the 54MHz reference whenever the carrier
    -- detect signal is negated.
    -- 
    SDI_CDR : xapp250_cdr 
        port map (
            vco             => rx_sclk,     
            refclk          => clk54M_ref,
            reset           => rst,
            data            => rx_ser_dat,
            force_resync    => not eq_cd,
            dataout         => dataout,
            vcoisfast       => open,
            vcoisslow       => open,
            mkvcofaster     => make_vco_faster,
            mkvcoslower     => make_vco_slower,
            vco_tc          => open,
            refclk_tc       => open,
            phasedet        => open,
            clkisfastc2d    => open,
            clkisslowc2d    => open);


    --
    -- Rx parallel clock generation
    --
    -- This code generates a 27MHz clock from the 270M from the CDR PLL. A 5-bit 
    -- ring counter is clocked by the 270M clock. The ring counter first fills up
    -- with all ones and then with all zeros, producing a 50% duty cycle 27MHz 
    -- clock.
    --
    RCNTR : process(rx_sclk, rst)
    begin
        if rst = '1' then
            ringcntr <= (others => '0');
        elsif rx_sclk'event and rx_sclk = '1' then
            ringcntr <= (ringcntr(3 downto 0) & not ringcntr(4));
        end if;
    end process;

    rxrecclk <= ringcntr(4);

    --
    -- Deserialization and synchronization registers
    --
    -- The ser2par_reg is a shift register that deserializes the data. The output
    -- of the ser2par_reg is loaded into the sync_reg at about the same time that
    -- as the falling edge of the rx_pclk. The data in the sync_reg is synchronized 
    -- to the rx_pclk domain by loading the contents of the sync_reg into the 
    -- desc_in_reg on the rising edge of rx_pclk. This gives us about 5 rx_sclk 
    -- cycles of setup time and hold time for the data going into the desc_in_reg, 
    -- minus clock distribution delays.
    --
    process(rx_sclk, rst)
    begin
        if rst = '1' then
            ser2par_reg <= (others => '0');
        elsif rx_sclk'event and rx_sclk = '1' then
            ser2par_reg <= (dataout(0) & ser2par_reg(9 downto 1));
        end if;
    end process;

    process(rx_sclk, rst)
    begin
        if rst = '1' then
            sync_ld <= '0';
        elsif rx_sclk'event and rx_sclk = '1' then
            if ringcntr(3) = '1' and ringcntr(2) = '0' then
                sync_ld <= '1';
            else
                sync_ld <= '0';
            end if;
        end if;
    end process;

    process(rx_sclk, rst)
    begin
        if rst = '1' then
            sync_reg <= (others => '0');
        elsif rx_sclk'event and rx_sclk = '1' then
            if sync_ld = '1' then
                sync_reg <= ser2par_reg;
            end if;
        end if;
    end process;

    process(rx_pclk, rst)
    begin
        if rst = '1' then
            desc_in_reg <= (others => '0');
        elsif rx_pclk'event and rx_pclk = '1' then
            desc_in_reg <= sync_reg;
        end if;
    end process;

    --
    -- par_descrambler
    --
    -- The SDI parallel descrambler decodes the bitstream. This module can be found 
    -- in XAPP288.
    --
    SDI_DESC : par_descrambler 
        port map (
            clk         => rx_pclk,
            rst         => rst,
            ld          => VCC,
            d           => desc_in_reg,
            q           => desc_out);

    --
    -- par_framer_mult
    --
    -- The framer synchronizes to TRS symbols and then frames each 10-bit video word
    -- appropriately. There are two versions of the parallel framer available in the
    -- XAPP288 reference design. One uses a MULT18X18 primitive as a barrel shifter,
    -- the other version implements the barrel shifter in logic. The version that 
    -- uses the MULT18X18 primitive saves some CLB logic when a spare MULT18X18 
    -- is available in the FPGA.
    --
    SDI_FRM : par_framer_mult
        port map (
            clk         => rx_pclk,
            rst         => rst,
            ce          => VCC,
            d           => desc_out,
            frame_en    => frame_en,
            q           => framer_out,
            trs         => framer_trs,
            nsp         => framer_nsp);

    frame_en <= framer_nsp; -- if using the EDH processor module, assign frame_en
                            -- to framer_nsp or ep_sync_switch where ep_sync_switch
                            -- is a signal from the EDH processor that indicates
                            -- when the current video is in the synchronous
                            -- switching interval

    --
    -- vid_reg
    --
    -- Holds the parallel data out of framer
    --
    process(rx_pclk, rst)
    begin
        if rst = '1' then
            vid_reg <= (others => '0');
        elsif rx_pclk'event and rx_pclk = '1' then
            vid_reg <= framer_out;
        end if;
    end process;

    rx_video <= vid_reg;



    --
    -- Output buffers for the video_clk and trs outputs
    --
    VIDCLK_OUT : OBUF_LVCMOS25 
        port map (
            I       => rx_pclk,
            O       => video_clk);

    TRS_OUT : OBUF_LVCMOS25 
        port map (
            I       => framer_trs,
            O       => trs);


    --
    -- Output buffers for the loop filter control signals
    -- 
    SDVCO_U : OBUF_LVDCI_33 
        port map (
            I       => make_vco_faster, 
            O       => sdi_vco_up);

    SDVCO_D : OBUF_LVDCI_33 
        port map (
            I       => make_vco_slower, 
            O       => sdi_vco_down);

end synth;
