//------------------------------------------------------------------------------
// par_framer_mult.v
//
// SMPTE 259M-1997 Parallel Framer Module
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2001 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Jul. 20, 2001
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//          Aug. 16, 2001     Creation
//
//
// Other modules instanced in this design:
//
//          None.
//------------------------------------------------------------------------------
/*
Description of module:

SMPTE 259M-1997 serial digital interface (SDI) is a standard for transmitting
digital video over a serial link.

This module accepts 10-bit parallel "unframed" data words from the SDI
descrambler module and examines the bit stream for the 30-bit TRS pramble. Once
a TRS is found, the framer then knows the bit boundary of all subsequent 10-bit
characters in the bit stream and uses this offset to generate a 10-bit parallel
data word on its output.

The module has the following control inputs:

ce: The clock enable input controls loading of all registers in the module. It
must be asserted whenever a new 10-bit word is to be loaded into the module. By
providing a clock enable, this module can use a clock that is running at the
bit rate of the SDI bit stream if ce is asserted once every ten clock cycles.

frame_en: This input controls whether the framer rsynchronize to new character
offsets when out-of-phase TRS symbols are detected. When this input is high,
out-of-phase TRS symbols will cause the framer to resynchronize.

The module generates the following outputs:

q: This 10-bit output port contains the properly framed data word. The data is
normally valid on this port for 10 clock cycles.

trs: (timing reference signal) This output is asserted when the q output
register holds any of the four words of a TRS.

nsp: (new start position) If frame_en is low and a TRS is detected that does not
match the current character offset, this signal will be asserted high. The nsp
signal will remain asserted until the offset error has been corrected..

There are normally three ways to use the frame_en input:

frame_en tied high: When frame_en is tied high, the framer will resynchronize
on every TRS detected. 

frame_en tied to nsp: When in this mode, the framer implements TRS filtering.
If a TRS is detected that is out of phase with the existing character offset,
nsp will be asserted, but the framer will not resynchronize. If the next TRS
received is in phase with the current character offset, nsp will go low and the
will not resynchronize. If the next TRS arrives out of phase with the current
character offset, then the new character offset will be loaded and nsp will be
deasserted. Single erroneous TRS  are ignored in this mode, but if they persist,
the decoder will adjust.

frame_en tied low: The automatic framing function is disabled when frame_en is
tied low. If data is being sent across the interface that does not comply with
the SDI standard and may contain data that looks like TRS symbols, the framing
function can be disabled in this manner.

This implementation uses a multiplier for the barrel shifter.
--------------------------------------------------------------------------------
*/

`timescale 1ns / 1ns

module par_framer_mult (
    clk,                            // input clock
    rst,                            // reset signal
    ce,                             // clock enable
    d,                              // input data
    frame_en,                       // enables resynchronization when high
    q,                              // output data
    trs,                            // asserted when out reg contains a TRS character
    nsp                             // new start position detected
);

// IO definitions   
input               clk;
input               rst;
input               ce;
input   [9:0]       d;
input               frame_en;
output  [9:0]       q;
output              trs;
output              nsp;

// Internal registers
reg     [9:0]       in1_reg;        // input register 1 
reg     [9:0]       in2_reg;        // input register 2
reg     [9:0]       in3_reg;        // input register 3
reg     [9:0]       offset_reg;     // offset register
reg     [18:0]      barrel_in;      // input register for the barrel shifter
reg                 nsp;            // nsp output register
reg     [3:0]       trs_out;        // used to generate the trs output signal

// Internal signals
wire    [38:0]      in_vector;      // concatenation of the four input registers
reg     [9:0]       trs_match1;     // which offsets in in_vector[18:0] match 0x3ff
reg     [9:0]       trs_match2;     // which offsets in in_vector[28:10] match 0x000
reg     [9:0]       trs_match3;     // which offsets in in_vector[38:29] match 0x000
wire    [9:0]       trs_match_all;  // which offsets match complete 30-bit TRS symbol
reg     [15:0]      trs_match1_l1;  // intermediate level of gate outputs in TRS detector
reg     [15:0]      trs_match2_l1;  // intermediate level of gate outputs in TRS detector
reg     [15:0]      trs_match3_l1;  // intermediate level of gate outputs in TRS detector
wire                trs_detected;   // asserted when TRS symbol is detected
reg                 trs_error;      // more than one offset matched the TRS symbol
reg     [9:0]       offset_val;     // the calculated offset value to load into offset_reg
wire    [35:0]      barrel_out;     // output of barrel shifter
wire                new_offset;     // mismatch between offset_val and offset_reg
wire                barrel_msb;     // msb of barrel shifter

integer             i;              // for loop variables

//
// Input registers 1, 2, and 3
//
// The following three registers form a 3-character deep pipeline
// which will be examined by the 39-bit wide TRS detector.
//
always @ (posedge clk or posedge rst)
    if (rst)
        begin
            in1_reg <= 0;
            in2_reg <= 0;
            in3_reg <= 0;
        end
    else if (ce)
        begin
            in1_reg <= d;
            in2_reg <= in1_reg;
            in3_reg <= in2_reg;
        end
    
//
// TRS detector
//
// The TRS detector finds 30-bit TRS preambles (0x3ff, 0x000, 0x000) in the
// input data stream. The TRS detector scans a 39-bit input vector
// consisting of all the bits from the three input registers plus the LS
// 9 bits of the d input data.
//
// The detector consists two main parts. 
//
// The first part is a series 10-bit AND and NOR gates that examine each
// possible bit location in the 39 input vector for the TRS preamble. These
// 10-bit wide AND and NOR gates have been coded here as two levels of
// 3 and 4 input gates because this results in a more compact implementation
// in most synthesis engines. 
//
// The outputs of these gates are assigned to the vectors trs_match1, 2, 
// and 3. These three vectors each contain 10 unary bits which indicate which 
// offset(s) matched the pattern being detected. ANDing these three vectors
// together generates another 10-bit vector called trs_match_all whose bits
// indicate which offset(s) matches the entire 30-bit TRS preamble.
//
// The second part of the TRS detector asserts the trs_detected signal whenever
// at least one bit in trs_match_all is asserted. If more than one bit in
// trs_match_all is asserted, then the trs_error signal is asserted. The
// offset_val vector is generated by reversing the order of the bits in 
// trs_match_all.
// 
assign in_vector = {d[8:0], in1_reg, in2_reg, in3_reg};

// first level of gates

always @ (in_vector)
    begin
        trs_match1_l1[ 0] <=  &in_vector[ 3: 0];
        trs_match1_l1[ 1] <=  &in_vector[ 4: 1];
        trs_match1_l1[ 2] <=  &in_vector[ 5: 2];
        trs_match1_l1[ 3] <=  &in_vector[ 6: 3];
        trs_match1_l1[ 4] <=  &in_vector[ 7: 4];
        trs_match1_l1[ 5] <=  &in_vector[ 8: 5];
        trs_match1_l1[ 6] <=  &in_vector[ 9: 6];
        trs_match1_l1[ 7] <=  &in_vector[10: 7];
        trs_match1_l1[ 8] <=  &in_vector[11: 8];
        trs_match1_l1[ 9] <=  &in_vector[12: 9];
        trs_match1_l1[10] <=  &in_vector[13:10];
        trs_match1_l1[11] <=  &in_vector[14:11];
        trs_match1_l1[12] <=  &in_vector[15:12];
        trs_match1_l1[13] <=  &in_vector[16:13];
        trs_match1_l1[14] <=  &in_vector[17:14];
        trs_match1_l1[15] <=  &in_vector[18:15];

        trs_match2_l1[ 0] <= ~|in_vector[13:10];
        trs_match2_l1[ 1] <= ~|in_vector[14:11];
        trs_match2_l1[ 2] <= ~|in_vector[15:12];
        trs_match2_l1[ 3] <= ~|in_vector[16:13];
        trs_match2_l1[ 4] <= ~|in_vector[17:14];
        trs_match2_l1[ 5] <= ~|in_vector[18:15];
        trs_match2_l1[ 6] <= ~|in_vector[19:16];
        trs_match2_l1[ 7] <= ~|in_vector[20:17];
        trs_match2_l1[ 8] <= ~|in_vector[21:18];
        trs_match2_l1[ 9] <= ~|in_vector[22:19];
        trs_match2_l1[10] <= ~|in_vector[23:20];
        trs_match2_l1[11] <= ~|in_vector[24:21];
        trs_match2_l1[12] <= ~|in_vector[25:22];
        trs_match2_l1[13] <= ~|in_vector[26:23];
        trs_match2_l1[14] <= ~|in_vector[27:24];
        trs_match2_l1[15] <= ~|in_vector[28:25];

        trs_match3_l1[ 0] <= ~|in_vector[23:20];
        trs_match3_l1[ 1] <= ~|in_vector[24:21];
        trs_match3_l1[ 2] <= ~|in_vector[25:22];
        trs_match3_l1[ 3] <= ~|in_vector[26:23];
        trs_match3_l1[ 4] <= ~|in_vector[27:24];
        trs_match3_l1[ 5] <= ~|in_vector[28:25];
        trs_match3_l1[ 6] <= ~|in_vector[29:26];
        trs_match3_l1[ 7] <= ~|in_vector[30:27];
        trs_match3_l1[ 8] <= ~|in_vector[31:28];
        trs_match3_l1[ 9] <= ~|in_vector[32:29];
        trs_match3_l1[10] <= ~|in_vector[33:30];
        trs_match3_l1[11] <= ~|in_vector[34:31];
        trs_match3_l1[12] <= ~|in_vector[35:32];
        trs_match3_l1[13] <= ~|in_vector[36:33];
        trs_match3_l1[14] <= ~|in_vector[37:34];
        trs_match3_l1[15] <= ~|in_vector[38:35];
    end

// second level of gates

always @ (trs_match1_l1)
    begin
        trs_match1[0] <= trs_match1_l1[ 0] & trs_match1_l1[ 4] & trs_match1_l1[ 6];
        trs_match1[1] <= trs_match1_l1[ 1] & trs_match1_l1[ 5] & trs_match1_l1[ 7];
        trs_match1[2] <= trs_match1_l1[ 2] & trs_match1_l1[ 6] & trs_match1_l1[ 8];
        trs_match1[3] <= trs_match1_l1[ 3] & trs_match1_l1[ 7] & trs_match1_l1[ 9];
        trs_match1[4] <= trs_match1_l1[ 4] & trs_match1_l1[ 8] & trs_match1_l1[10];
        trs_match1[5] <= trs_match1_l1[ 5] & trs_match1_l1[ 9] & trs_match1_l1[11];
        trs_match1[6] <= trs_match1_l1[ 6] & trs_match1_l1[10] & trs_match1_l1[12];
        trs_match1[7] <= trs_match1_l1[ 7] & trs_match1_l1[11] & trs_match1_l1[13];
        trs_match1[8] <= trs_match1_l1[ 8] & trs_match1_l1[12] & trs_match1_l1[14];
        trs_match1[9] <= trs_match1_l1[ 9] & trs_match1_l1[13] & trs_match1_l1[15];
    end

always @ (trs_match2_l1)
    begin
        trs_match2[0] <= trs_match2_l1[ 0] & trs_match2_l1[ 4] & trs_match2_l1[ 6];
        trs_match2[1] <= trs_match2_l1[ 1] & trs_match2_l1[ 5] & trs_match2_l1[ 7];
        trs_match2[2] <= trs_match2_l1[ 2] & trs_match2_l1[ 6] & trs_match2_l1[ 8];
        trs_match2[3] <= trs_match2_l1[ 3] & trs_match2_l1[ 7] & trs_match2_l1[ 9];
        trs_match2[4] <= trs_match2_l1[ 4] & trs_match2_l1[ 8] & trs_match2_l1[10];
        trs_match2[5] <= trs_match2_l1[ 5] & trs_match2_l1[ 9] & trs_match2_l1[11];
        trs_match2[6] <= trs_match2_l1[ 6] & trs_match2_l1[10] & trs_match2_l1[12];
        trs_match2[7] <= trs_match2_l1[ 7] & trs_match2_l1[11] & trs_match2_l1[13];
        trs_match2[8] <= trs_match2_l1[ 8] & trs_match2_l1[12] & trs_match2_l1[14];
        trs_match2[9] <= trs_match2_l1[ 9] & trs_match2_l1[13] & trs_match2_l1[15];
    end

always @ (trs_match3_l1)
    begin
        trs_match3[0] <= trs_match3_l1[ 0] & trs_match3_l1[ 4] & trs_match3_l1[ 6];
        trs_match3[1] <= trs_match3_l1[ 1] & trs_match3_l1[ 5] & trs_match3_l1[ 7];
        trs_match3[2] <= trs_match3_l1[ 2] & trs_match3_l1[ 6] & trs_match3_l1[ 8];
        trs_match3[3] <= trs_match3_l1[ 3] & trs_match3_l1[ 7] & trs_match3_l1[ 9];
        trs_match3[4] <= trs_match3_l1[ 4] & trs_match3_l1[ 8] & trs_match3_l1[10];
        trs_match3[5] <= trs_match3_l1[ 5] & trs_match3_l1[ 9] & trs_match3_l1[11];
        trs_match3[6] <= trs_match3_l1[ 6] & trs_match3_l1[10] & trs_match3_l1[12];
        trs_match3[7] <= trs_match3_l1[ 7] & trs_match3_l1[11] & trs_match3_l1[13];
        trs_match3[8] <= trs_match3_l1[ 8] & trs_match3_l1[12] & trs_match3_l1[14];
        trs_match3[9] <= trs_match3_l1[ 9] & trs_match3_l1[13] & trs_match3_l1[15];
    end

// third level of gates generates a unary bit pattern indicating which offsets
// contain valid TRS symbols
assign trs_match_all = trs_match1 & trs_match2 & trs_match3;

// If any of the bits in trs_match_all are asserted, the assert trs_detected        
assign trs_detected = |trs_match_all;

// The case statement determines if more than one bit is set, asserting trs_error
// if so. The for loop generates the offset_val vector by reversing the bit
// order of trs_match_all.
always @ (trs_match_all)
    begin
        case (trs_match_all)                
            10'b00_0000_0000: trs_error  <= 0;
            10'b00_0000_0001: trs_error  <= 0;
            10'b00_0000_0010: trs_error  <= 0;
            10'b00_0000_0100: trs_error  <= 0;
            10'b00_0000_1000: trs_error  <= 0;
            10'b00_0001_0000: trs_error  <= 0;
            10'b00_0010_0000: trs_error  <= 0;
            10'b00_0100_0000: trs_error  <= 0;
            10'b00_1000_0000: trs_error  <= 0;
            10'b01_0000_0000: trs_error  <= 0;
            10'b10_0000_0000: trs_error  <= 0;
            default:          trs_error  <= 1;
        endcase

        for (i = 0; i < 10; i = i + 1)
            offset_val[i] <= trs_match_all[9 - i];
    end

//
// offset_reg: barrel shifter offset register
//
// The offset_reg loads the offset_val whenever trs_detected is
// asserted and trs_error is not asserted and frame_en is asserted.
//
always @ (posedge clk or posedge rst)
    if (rst)
        offset_reg <= 0;
    else if (ce) 
        if (trs_detected & ~trs_error & frame_en)
            offset_reg <= offset_val;

//
// New start position detector
// 
// A comparison between offset_val and offset_reg determines if
// the new offset is different than the current one. If there is
// a mismatch and frame_en is not asserted, then the nsp output
// will be asserted.
//
assign new_offset = offset_val != offset_reg;

always @ (posedge clk or posedge rst)
    if (rst)
        nsp <= 1;
    else if (ce)        
        if (trs_detected & ~trs_error)
            nsp <= ~frame_en & new_offset;

//
// barrel_in: barrel shifter input register
//
// This register implements a pipeline delay stage so that the
// barrel shifter's data input matches the delay on the offset
// input caused by the offset_reg.
//
always @ (posedge clk or posedge rst)
    if (rst)
        barrel_in <= 0;
    else if (ce)
        barrel_in <= {in2_reg[8:0], in3_reg};

//
// barrel shifter
//
// The barrel shifter extracts a 10-bit field from the 19-bit barrel_in vector.
// The bits extracted depend on the value of the offset_reg. If offset_reg is
// zero, barrel_in[9:0] are output by the barrel shifter. At the other extreme,
// an offset_reg value of 9 causes the barrel shifter to output barrel_in[18:9].
//
// In this implementation, the nine LS bits of the barrel shifter are implemented
// with a Virtex II embedded 18x18 multiplier. The LS 18-bits of barrel_in are
// connected to the A input of the multiplier. The B input of the multiplier is
// connected to a shiftcode value derived from the value in the offset_reg. The
// shiftcode is a 10-bit unary bit code. Since only one bit of shiftcode is
// set at a time, the shiftcode essentially multiplies the barrel_in value
// by a power of two from 1 (when the offset_reg value is 9) to 512 (when the
// offset_reg value is 0).
//
// The barrel shifter's outputs are taken from bits [17:9] of the multiplier's
// product output port. When the offset_reg contains a value of 9, the
// barrel_in vector is multiplied by a shiftcode of 1, causing bits [17:9] of
// barrel_in to pass directly to the multiplier's output bits [17:9]. At the
// other extreme, when the offset_reg contains a value of 0, the barrel_in
// vector is multiplied by a shiftcode of 512, causing bits [8:0] of barrel_in
// to be shifted up so they appear on the multiplier's output bits [17:9].
//
// The MS bit of the barrel shifter is produced with a 10:1 MUX using a case
// statement.
//

//
// Instantiate the 18x18 multiplier.
//
MULT18X18 BS (
    .A (barrel_in[17:0]),
    .B ({8'b0000_0000, offset_reg}),
    .P (barrel_out)
);

//
// Generate a 10:1 MUX for the MSB of the barrel shifter.
//
assign barrel_msb = (barrel_in[ 9] & offset_reg[9]) |
                    (barrel_in[10] & offset_reg[8]) |
                    (barrel_in[11] & offset_reg[7]) |
                    (barrel_in[12] & offset_reg[6]) |
                    (barrel_in[13] & offset_reg[5]) |
                    (barrel_in[14] & offset_reg[4]) |
                    (barrel_in[15] & offset_reg[3]) |
                    (barrel_in[16] & offset_reg[2]) |
                    (barrel_in[17] & offset_reg[1]) |
                    (barrel_in[18] & offset_reg[0]);

//
// Use bits [17:9] of the multiplier's output as the barrel shifter output.
//
assign q = {barrel_msb, barrel_out[17:9]};

//
// trs: trs output generation logic
//
// The trs_out register is a 4-bit shift register which shifts everytime
// the bit_cntr[0] bit is asserted. The trs output signal is the OR of
// the four bits in this register so it becomes asserted when the first
// character of the TRS symbol is output and remains asserted for the
// following three characters of the TRS symbol.
//
always @ (posedge clk or posedge rst)
    if (rst)
        trs_out <= 0;
    else if (ce)
        trs_out <= {trs_detected, trs_out[3:1]};

assign trs = |trs_out;

endmodule
