//-----------------------------------------------------------------
// par_scrambler.v
//
// SDI Parallel Scrambler module
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2001 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Aug. 24, 2001
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//          Aug. 24, 2001     Creation
//
//
// Other modules instanced in this design:
//
//          None.
//-----------------------------------------------------------------
/*
Description of module:

SMPTE 259M-1997 serial digital interface (SDI) is a standard for transmitting
digital video over a serial link.

SDI specifies that the serial bit stream shall be encoded in two ways. First, a
generator polynomial of x^9 + x^4 + 1 is used to generate a scrambled NRZ bit
stream. Next, a generator polynomial of x + 1 is used to produce the final
polarity free NRZI bit stream which is transmitted over the physical layer.

The scrambler module described in this file sits at the transmitting end of the
SDI link and divides the video bit stream by the two generator polynomials.

The nrzi input, when high, enables the NRZ-to-NRZI conversion. If this input is
low, this conversion is bypassed.

The scram input, when high, enables the SDI scrambler. If this input is low, the
scrambler is bypassed.

This module is a parallel implementation of the scrambler. It accepts one 10-bit
data word in every clock cycle and generates one 10-bit word out every clock
cycle. The ld signal causes all registers in the module to load and serves as
a clock enable for the module. 
--------------------------------------------------------------------------------
*/

`timescale 1ns / 1ns

module par_scrambler (
    clk,                        // input clock (bit-rate)
    rst,                        // reset signal
    ld,                         // input register load signal
    nrzi,                       // enables NRZ-to-NRZI conversion when high
    scram,                      // enables SDI scrambler when high
    d,                          // input data 
    q                           // output data
);

// IO definitions   
input               clk;
input               rst;
input               ld;
input               nrzi;
input               scram; 
input   [9:0]       d;
output  [9:0]       q;

// Internal registers
reg     [9:0]       scram_reg;  // pipeline delay register
reg     [9:0]       out_reg;    // output register

// Internal wires
wire    [13:0]      scram_in;   // input to the scambler
reg     [9:0]       scram_temp; // intermediate output of the scrambler
wire    [9:0]       scram_out;  // output of the scrambler
wire    [9:0]       nrzi_out;   // output of NRZ-to-NRZI converter
reg     [9:0]       nrzi_temp;  // intermediate output of the NRZ-to-NRZI converter

integer             i, j;       // for loop variables


//
// SDI Scrambler
//
// This block of logic implements the SDI scrambler algorithm. The scrambler
// uses the 10 incoming bits from the input port and a 14-bit vector called 
// scram_in. scram_in is made up of 9 bits that were scrambled in the previous 
// clock cycle and are stored in the 9 MSBs of scram_reg and the 5 LS scrambled 
// bits that have been generated during the current clock cycle. The results of 
// the scrambler are assigned to scram_temp.
//
// A MUX will output either the value of scram_temp or the d input word
// depending on the scram enable input. The output of the MUX is stored in the
// scram_reg.
//
assign scram_in = {scram_temp[4:0], scram_reg[9:1]};

always @ (d or scram_in)
    for (i = 0; i < 10; i = i + 1)
        scram_temp[i] = d[i] ^ scram_in[i] ^ scram_in[i + 4];

assign scram_out = scram ? scram_temp : d;

always @ (posedge clk or posedge rst)
    if (rst)
        scram_reg <= 0;
    else if (ld)
        scram_reg <= scram_out;

//
// NRZ-to-NRZI converter
//
// This block of logic implements the NRZ-to-NRZI conversion. It operates on the
// 10 bits coming from the scram_reg and the MSB stored in the out_reg. A MUX
// bypasses the conversion process if the nrzi input is low.
//
always @ (scram_reg or out_reg or nrzi_out)
    begin
        nrzi_temp[0] = out_reg[9] ^ scram_reg[0];
        for (j = 1; j < 10; j = j + 1)
            nrzi_temp[j] = nrzi_out[j - 1] ^ scram_reg[j];
    end

assign nrzi_out = nrzi ? nrzi_temp : scram_reg;

//
// out_reg: Output register
//
always @ (posedge clk or posedge rst)
    if (rst)
        out_reg <= 0;
    else if (ld)
        out_reg <= nrzi_out;

assign q = out_reg;

endmodule
