//------------------------------------------------------------------------------
// pd_clk2data22.v
//
// Phase detector and data recovery unit for XAPP250 CDR
//
//
//
//                  Author: Leonard Dieguez
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Oct. 28, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//      Oct 28, 2003        Release 1.0
//                                                      
// Other modules instanced in this design:
//      muxdelchain32
//
//------------------------------------------------------------------------------
/*
Description of module:

This module contains the phase detector used to lock the VCO to the frequency
of the incoming bitstream. It also contains the data recovery unit.

--------------------------------------------------------------------------------
*/

module  pd_clk2data22 (
    //inputs
    clk,
    reset,
    data,
    //outputs
    data_out,
    mkclkfaster,
    mkclkslower,
    //test only
    phasedet_exp,
    clklags_exp,
    clkleads_exp,   
    clkisfast_exp,
    clkisslow_exp
    );

///////////////////////////////////////////////////
// port declarations
input   clk;
input   reset;
input   data;


output  [1:0] data_out;
output  mkclkfaster;
output  mkclkslower;

//test signals used for debug only
output  [15:0] phasedet_exp;    //phase detection for export (debug)
output  [3:0]   clklags_exp;
output  [3:0]   clkleads_exp;
output      clkisfast_exp;
output      clkisslow_exp;

///////////////////////////////////////////////////
reg [31:0]  pd_2R;


reg [15:0]  phase;
reg [15:0]  phaseR;
reg [15:0]  phase2R;
reg [15:0]  phase3R;

reg     phase_en /* synthesis syn_direct_enable = 1 syn_keep=1*/;

wire    [31:0]  data_taps;

//test signals

////////////////////////////////////////////////////////////////
//This is the main delay chain used for phase detection
muxdelchain32 pddelay(
    .clk(clk),
    .reset(reset),
    .cin(data),
    .out(data_taps),
    .data(data_out)
);


reg [11:0]  clkcount;
reg [3:0]   clkcntlow;
wire        clk_tc       /* synthesis syn_direct_enable = 1 syn_keep=1*/;
wire        clkcntlow_tc /* synthesis syn_direct_enable = 1 syn_keep=1*/;

assign clkcntlow_tc = clkcntlow[3];
assign clk_tc = clkcount[11];

///////////////////////////////////////////////////////////////////
//divide clk
///////////////////////////////////////////////////////////////////
//divide counter,  
always @ (posedge clk or posedge reset ) begin
    if (reset) begin
        clkcount    <= #1 12'b0;
        //pd_enable <= #1 1'b0;
    end
    else if( clkcntlow_tc ) begin
        clkcount    <= #1 clkcount + 1;
        //pd_enable <= #1 1'b0;
    end
end

////////////////////////////////////////////////////////////////////
//lower 2 bits of the vco counter.
//combined reset is   (0x29) 
//so the low 2 bits reset to 3
//shift reg counter is one hot code;
//bit 0 is 2'b00
//bit 1 is 2'b01
//bit 2 is 2'b10
//bit 3 is 2'b11
always @ (posedge clk or posedge reset) begin 
    if (reset) begin
        clkcntlow   <= #1 4'b0001;
    end
    else begin
        clkcntlow   <= #1 {clkcntlow[2:0], clkcntlow[3]};
    end
end

////////////////////////////////////////////////////////////////////

reg [11:0]  pd_enable /* synthesis syn_direct_enable = 1 syn_keep=1 */;
////////////////////////////////////////////////////////////////////
// pd_enable is used to enable clk(VCO) updates. 
// pd_enable[0] can be used as a clock enable that is active every 
// 4 clocks VCO phase updates are governed by the rate of 
// pd_enable bit that is used.
always @ (posedge clk or posedge reset ) begin 
    if (reset) begin
        pd_enable   <= #1 12'b0 ;
    end
    else begin 
        //div by 4
        pd_enable[0]    <= #1 clkcntlow_tc ;
        //div by 8
        pd_enable[1]    <= #1 clkcntlow_tc & clkcount[0];
        //div by 16 
        pd_enable[2]    <= #1 pd_enable[1] & clkcount[1] ;
        //div by 32
        pd_enable[3]    <= #1 pd_enable[2] & clkcount[2] ;
        //div by 64
        pd_enable[4]    <= #1 pd_enable[3] & clkcount[3] ;
        //div by 128 
        pd_enable[5]    <= #1 pd_enable[4] & clkcount[4] ;
        //div by 256 
        pd_enable[6]    <= #1 pd_enable[5] & clkcount[5] ;
        //div by 512 
        pd_enable[7]    <= #1 pd_enable[6] & clkcount[6] ;
        //div by 1024
        pd_enable[8]    <= #1 pd_enable[7] & clkcount[7] ;
        //div by 2048
        pd_enable[9]    <= #1 pd_enable[8] & clkcount[8] ;
        //div by 4096 
        pd_enable[10]   <= #1 pd_enable[9] & clkcount[9] ;
        //div by 8192 
        pd_enable[11]   <= #1 pd_enable[10] & clkcount[10] ;
    end
end


//////////////////////////////////////////////
//register the delayed data (taps)
//part of the phase detector for use with the VCO
//implimentation.
always @ (posedge clk or posedge reset ) begin 
    if (reset) begin
        pd_2R       <= #1 32'b0 ;
        phase       <= #1 16'b0 ;
        phase_en    <= #1 1'b0  ;
        phaseR      <= #1 16'b0 ;
        phase2R     <= #1 16'b0 ;
        phase3R     <= #1 16'b0 ;
        end
        
    else  begin
        pd_2R       <= #1 data_taps;
        
        //detect a trasition during this period.
        phase_en    <= #1   pd_2R[31] & ~pd_2R[0] ;

        //detect the rising edge
        //as it travels up the delay chain
        phase[0]    <= #1   pd_2R[2] & ~pd_2R[0] ;
        phase[1]    <= #1   pd_2R[4] & ~pd_2R[2] ;  
        phase[2]    <= #1   pd_2R[6] & ~pd_2R[4] ;
        phase[3]    <= #1   pd_2R[8] & ~pd_2R[6] ;
        phase[4]    <= #1   pd_2R[10] & ~pd_2R[8] ;
        phase[5]    <= #1   pd_2R[12] & ~pd_2R[10] ;
        phase[6]    <= #1   pd_2R[14] & ~pd_2R[12] ;
        phase[7]    <= #1   pd_2R[16] & ~pd_2R[14] ;
        
        //center.
        phase[8]    <= #1   pd_2R[17] & ~pd_2R[16] ;
        phase[9]    <= #1   pd_2R[19] & ~pd_2R[17] ;
        phase[10]   <= #1   pd_2R[21] & ~pd_2R[19] ;
        phase[11]   <= #1   pd_2R[23] & ~pd_2R[21] ;
        phase[12]   <= #1   pd_2R[25] & ~pd_2R[23] ;
        phase[13]   <= #1   pd_2R[27] & ~pd_2R[25] ;
        phase[14]   <= #1   pd_2R[29] & ~pd_2R[27] ;
        phase[15]   <= #1   pd_2R[31] & ~pd_2R[29] ;
        
        //only update when a valid phase hit occurs
        if (phase_en) begin
            phaseR      <= #1 phase;
        end 
        //store phaseR for one clock
        if(pd_enable[0]) begin
            phase2R     <= #1 phaseR;
            phase3R     <= #1 phase2R;
        end
        
    end
end


//////////////////////////////////////////////////////////////////////////////
// this may be implimented in a pipline version for full rate operation
// phase encoder and averaging circuit. Not used in this file 
// but user may want to try for VCO control of phase alignment.
wire    [3:0]   phase_code ;
wire    [3:0]   phase_ave ;

//////////////////////////////////////////////////////////////////
reg [14:0]  clkisfast ;
reg [14:0]  clkisslow ;
reg     clkisfast1 ;
reg     clkisslow1 ;
reg     clkisfast_en ;
reg         clkisslow_en ;
reg     clkisfast_enR ;
reg     clkisslow_enR ;
/////////////////////////////////////////////////////////////////
// frequency detection
// frequency detection
// dectection for clk is fast does not occur if the phase hit is at the end of the 
// delay line and in moving towards the center.
//
// dectection for clk is slow does not occur if the phase hit is at the start of the 
// delay line and is moving towards the center.
//
// in other words do not correct phase movement that is moving towards where you want
// the phase hits to be.

always @ (posedge clk or posedge reset ) begin 
    if (reset) begin
        clkisslow   <= #1 15'b0;
        clkisfast   <= #1 15'b0;
        clkisfast1  <= #1 1'b0;
        clkisslow1  <= #1 1'b0;
        clkisfast_en    <= #1 1'b0 ;
        clkisslow_en    <= #1 1'b0 ;
        clkisfast_enR   <= #1 1'b0 ;
        clkisslow_enR   <= #1 1'b0 ;
        end
        
    else if (pd_enable[0]) begin
        //clock is fast                      
        clkisfast[14]   <= #1   (phase2R[14] & phase3R[15] );
                                          
        clkisfast[13]   <= #1   (phase2R[13] & phase3R[14] ) |
                    (phase2R[13] & phase3R[15] );
                                          
        clkisfast[12]   <= #1   (phase2R[12] & phase3R[13] ) |
                    (phase2R[12] & phase3R[14] ) |
                    (phase2R[12] & phase3R[15] );
                                          
        clkisfast[11]   <= #1   (phase2R[11] & phase3R[12] ) |
                    (phase2R[11] & phase3R[13] ) |
                    (phase2R[11] & phase3R[14] );
                                          
        clkisfast[10]   <= #1   (phase2R[10] & phase3R[11] ) |
                    (phase2R[10] & phase3R[12] ) |
                    (phase2R[10] & phase3R[13] );
                                          
        clkisfast[9]    <= #1   (phase2R[9]  & phase3R[10] ) |
                    (phase2R[9]  & phase3R[11] ) |
                    (phase2R[9]  & phase3R[12] );
                                          
        clkisfast[8]    <= #1   (phase2R[8]  & phase3R[9] ) |
                    (phase2R[8]  & phase3R[10] ) |
                    (phase2R[8]  & phase3R[11] );
                                                              
        clkisfast[7]    <= #1   (phase2R[7]  & phase3R[8] ) |
                    (phase2R[7]  & phase3R[9] ) |
                    (phase2R[7]  & phase3R[10]);
                                          
        clkisfast[6]    <= #1   (phase2R[6]  & phase3R[7] ) |
                    (phase2R[6]  & phase3R[8] ) |
                    (phase2R[6]  & phase3R[9] );
                                          
        clkisfast[5]    <= #1   (phase2R[5]  & phase3R[6] ) |
                    (phase2R[5]  & phase3R[7] ) |
                    (phase2R[5]  & phase3R[8] );
                                          
        clkisfast[4]    <= #1   (phase2R[4]  & phase3R[5] ) |
                    (phase2R[4]  & phase3R[6] ) |
                    (phase2R[4]  & phase3R[7] );
                                          
        clkisfast[3]    <= #1   (phase2R[3]  & phase3R[4] ) |
                    (phase2R[3]  & phase3R[5] ) |
                    (phase2R[3]  & phase3R[6] );
                                          
        clkisfast[2]    <= #1   (phase2R[2]  & phase3R[3] ) |
                    (phase2R[2]  & phase3R[4] ) |
                    (phase2R[2]  & phase3R[5] );
                                          
        clkisfast[1]    <= #1   (phase2R[1]  & phase3R[2] ) |
                    (phase2R[1]  & phase3R[3] ) |
                    (phase2R[1]  & phase3R[4] );
                                          
        clkisfast[0]    <= #1   (phase2R[0]  & phase3R[1] ) |
                    (phase2R[0]  & phase3R[2] ) |
                    (phase2R[0]  & phase3R[3] );        

        //clock is slow
        clkisslow[0]    <= #1   (phase2R[15] & phase3R[14]) |
                    (phase2R[15] & phase3R[13]) |
                    (phase2R[15] & phase3R[12]);
                                          
        clkisslow[1]    <= #1   (phase2R[14] & phase3R[13]) |
                    (phase2R[14] & phase3R[12]) |
                    (phase2R[14] & phase3R[11]);
                                          
        clkisslow[2]    <= #1   (phase2R[13] & phase3R[12]) |
                    (phase2R[13] & phase3R[11]) |
                    (phase2R[13] & phase3R[10]);
                                          
        clkisslow[3]    <= #1   (phase2R[12] & phase3R[11]) |
                    (phase2R[12] & phase3R[10]) |
                    (phase2R[12] & phase3R[ 9]);
                                          
        clkisslow[4]    <= #1   (phase2R[11] & phase3R[10]) |
                    (phase2R[11] & phase3R[ 9]) |
                    (phase2R[11] & phase3R[ 8]);
                                          
        clkisslow[5]    <= #1   (phase2R[10] & phase3R[9]) |
                    (phase2R[10] & phase3R[8]) |
                    (phase2R[10] & phase3R[7]);
                                         
        clkisslow[6]    <= #1   (phase2R[9]  & phase3R[8]) |
                    (phase2R[9]  & phase3R[7]) |
                    (phase2R[9]  & phase3R[6]);
                                                      
        clkisslow[7]    <= #1   (phase2R[8]  & phase3R[7]) |
                    (phase2R[8]  & phase3R[6]) |
                    (phase2R[8]  & phase3R[5]);
                                                             
        clkisslow[8]    <= #1   (phase2R[7]  & phase3R[6]) |
                    (phase2R[7]  & phase3R[5]) |
                    (phase2R[7]  & phase3R[4]);
                                           
        clkisslow[9]    <= #1   (phase2R[6]  & phase3R[5]) |
                    (phase2R[6]  & phase3R[4]) |
                    (phase2R[6]  & phase3R[3]);
                                           
        clkisslow[10]   <= #1   (phase2R[5]  & phase3R[4]) |
                    (phase2R[5]  & phase3R[3]) |
                    (phase2R[5]  & phase3R[2]);
                                           
        clkisslow[11]   <= #1   (phase2R[4]  & phase3R[3]) |
                    (phase2R[4]  & phase3R[2]) |
                    (phase2R[4]  & phase3R[1]);
                                           
        clkisslow[12]   <= #1   (phase2R[3]  & phase3R[2]) |
                    (phase2R[3]  & phase3R[1]) |
                    (phase2R[3]  & phase3R[0]);
                                           
        clkisslow[13]   <= #1   (phase2R[2]  & phase3R[1]) |
                    (phase2R[2]  & phase3R[0]);
                                         
        clkisslow[14]   <= #1   (phase2R[1]  & phase3R[0]);

    
        clkisfast_en    <= #1   |clkisfast[14:0] ;
        clkisslow_en    <= #1   |clkisslow[14:0] ;
        clkisfast_enR   <= #1   clkisfast_en ;
        clkisslow_enR   <= #1   clkisslow_en ;
        
        clkisfast1  <= #1   |clkisfast[14:7] ;
        clkisslow1  <= #1   |clkisslow[14:7] ;
        
        end
end


////////////////////////////////////////////////
reg [10:0]  cisfastcnt ;
reg [10:0]  cisslowcnt ;

reg     cisfast_tcR ;
reg     cisslow_tcR ;

wire        cisfast_tc ;
wire        cisslow_tc ;
wire        reversal_tc ;

wire        clkleads_tc ;
wire        clklags_tc ;


////////////////////////////////////////////////
//clock is fast or slow counters.
//this is the frequency detection filter.
//it will act as an integrator for the frequency differences
//in the ideal case when both the data and the oscillator are 
//frequency locked but not phase locked this counter should not
//have an output or be counting.
/////////////////////////////////////////////////
always @ (posedge clk or posedge reset ) begin 
    if (reset) begin
        cisfastcnt  <= #1 0 ;
        cisslowcnt  <= #1 0 ;
        cisfast_tcR <= #1 1'b0 ;
        cisslow_tcR <= #1 1'b0 ;
        end
    else if ( pd_enable[0] ) begin  //enable the counter every 4 clocks
        //////////////////////////////////////////////////////////
        if (cisfast_tc )    //use the msb as the sync 
                    //reset for both counters
            cisfastcnt  <= #1 0 ;
        //else if (clkisfast_en | clkleads_tc)
        else if (clkisfast_en )
            cisfastcnt  <= #1 cisfastcnt + 1 ;
        //////////////////////////////////////////////////////////
        if (cisslow_tc )    //use the msb as the sync 
                    //reset for both counters
            cisslowcnt  <= #1 0 ;   
        //else if (clkisslow_en | clklags_tc)
        else if (clkisslow_en )
            cisslowcnt  <= #1 cisslowcnt + 1 ;              
    end     
end

assign cisfast_tc = cisfastcnt[9] ; //assign the terminal counts
assign cisslow_tc = cisslowcnt[9] ;


/////////////////////////////////////////////////////////////////////////
reg     clklags ;
reg     clkleads ;

reg [10:0]  clkleadscnt ;
reg [10:0]  clklagscnt ;
reg     clkleads_tcR ;
reg     clklags_tcR ;

reg     mkclkfaster;
reg     mkclkslower;

reg [3:0]   mkclkfaster_ila;
reg [3:0]   mkclkslower_ila;

//////////////////////////////////////////////
//phaseR 8 to 15 clk lags
//phaseR 0 to 7  clk leads
//
// section to correct phase errors but not frequency error
// pd_enable is used to "weight" the correction values.
// for example, the farther from the center the error is 
// the more correction is imposed.
////////////////////////////////////////////////
//clock leads or lags counters.
//this is the phase detection filter.
//it will act as an integrator for the phase differences
//in the ideal case when both the data and the oscillator are 
//frequency locked but not phase locked this counter should output
//corrections for the phase error
/////////////////////////////////////////////////
always @ (posedge clk or posedge reset ) begin 
    if (reset) begin
        clklags     <= #1 0 ;
        clkleads    <= #1 0 ;
        clkleadscnt     <= #1 0 ;
        clklagscnt  <= #1 0 ;   
        end
        
    else if ( pd_enable[0] ) begin  //enable the counter every 4 clocks 

        //lead lag counters
        if (clkleads_tc) //use the msb as the sync reset for both counters
            clkleadscnt <= #1 0 ;

        else if (phase2R[0])
            clkleadscnt     <= #1 clkleadscnt + 8 ;
        else if (phase2R[1])
            clkleadscnt     <= #1 clkleadscnt + 7 ;
        else if (phase2R[2])
            clkleadscnt     <= #1 clkleadscnt + 6 ;
        else if (phase2R[3])
            clkleadscnt     <= #1 clkleadscnt + 6 ;
        else if (phase2R[4])
            clkleadscnt     <= #1 clkleadscnt + 4 ;
        else if (phase2R[5])
            clkleadscnt     <= #1 clkleadscnt + 3 ;
        else if (phase2R[6])
            clkleadscnt     <= #1 clkleadscnt + 2 ;
        else if (phase2R[7])
            clkleadscnt     <= #1 clkleadscnt + 1 ;
        
        ////////////////////////////////////////////////////////////////////
        if (clklags_tc) //use the msb as the sync reset for both counters
            clklagscnt  <= #1 0 ;
        else if (phase2R[8])
            clklagscnt  <= #1 clklagscnt + 1 ;
        else if (phase2R[9])
            clklagscnt  <= #1 clklagscnt + 2 ;
        else if (phase2R[10])
            clklagscnt  <= #1 clklagscnt + 3 ;
        else if (phase2R[11])
            clklagscnt  <= #1 clklagscnt + 4 ;
        else if (phase2R[12])
            clklagscnt  <= #1 clklagscnt + 5 ;
        else if (phase2R[13])
            clklagscnt  <= #1 clklagscnt + 6 ;
        else if (phase2R[14])
            clklagscnt  <= #1 clklagscnt + 7 ;
        else if (phase2R[15])
            clklagscnt  <= #1 clklagscnt + 8 ;
    end

end

assign clkleads_tc = clkleadscnt[9] ;   //assign the terminal counts
assign clklags_tc = clklagscnt[9] ;



always @ (posedge clk or posedge reset ) begin 
    if (reset) begin
        mkclkfaster <= #1 0 ;
        mkclkslower <= #1 0 ;
    end
        
    else  begin
        mkclkfaster <= #1 (clklags_tc) & pd_enable[0] ;
        mkclkslower <= #1 (clkleads_tc) & pd_enable[0] ;
    end
end



//////////////////////////////////////////////////////////////////////
// debug outputs only ;you can use this to drive chip scope or logic
// analyzer on external pins. some signals go to the LCD display.
reg [15:0]  phasedet_exp ;
reg [3:0]   clkleads_exp ;
reg [3:0]   clklags_exp ;
reg     clkisfast_exp ;
reg     clkisslow_exp ;

always @ (posedge clk or posedge reset ) begin 
    if (reset) begin
        phasedet_exp    <= #1 16'b0 ;
        clkleads_exp    <= #1 4'b0 ;
        clklags_exp <= #1 4'b0 ;
        clkisfast_exp   <= #1 1'b0 ;
        clkisslow_exp   <= #1 1'b0 ;
    end
        
    else if (pd_enable[0]) begin
        phasedet_exp    <= #1 phase2R ;
    end
end

endmodule

