//-----------------------------------------------------------------------------
// sdi_tx_ddr.v
//
// SDI transmitter module
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Oct. 28, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//          Oct. 28, 2003     Release 1.0
//
//
// Other modules instanced in this design:
//          par_scrambler (from XAPP298)
//
//-----------------------------------------------------------------------------
/*
Module description:

This module instantiates the SDI scrambler, par_scrambler, and implements the
serializer. The scrambler runs at the word rate, encoding 10-bits every cycle
of the 27MHz word clock. The serializer loads once every 5 cycles of the
135MHz clock from the output of the scrambler module. The serializer produces
two bits every cycle of the 135MHz clock to feed a FDDRRSE primitive.
*/
`timescale 1ns / 1ps

module sdi_tx_ddr (
    sclk,
    pclk,
    rst,
    vid_in,
    q
);

input                   sclk;           // half bit-rate clock
input                   pclk;           // word rate clock
input                   rst;            // reset input
input   [9:0]           vid_in;         // parallel unscrambled data in
output  [1:0]           q;              // encoded serial data output bits


// Internal signal definitions
wire                    VCC;
wire                    GND;
wire    [9:0]           scram_vid;      // video from SDI encoder
reg     [9:0]           sync_reg;       // synchronizes video between pclk and sclk domains
reg     [9:0]           par2ser;        // serializer
wire                    par2ser_ld;     // load enable to serializer
reg     [2:0]           ld_gen;         // load enable generator

assign VCC = 1'b1;
assign GND = 1'b0;



//----------------------------------------------------------------------------
// SMPTE 259M encoder module
//
// This module encodes the video per the SMPTE 259M standard.
//
par_scrambler SCRAM (
    .clk            (pclk),
    .rst            (rst),
    .ld             (VCC),      // load every clock cycle
    .nrzi           (VCC),      // enable the NRZ-to-NRZI converter
    .scram          (VCC),      // enable the SMPTE scrambler
    .d              (vid_in),
    .q              (scram_vid));
     
//----------------------------------------------------------------------------
// Synchronization and serialization
//

//
// Sync register
//
// This register holds the output of the scrambler module. It clocks 
// synchronously with the word-rate clock, but its output feeds the serializer
// register that clocks synchronously with the sclk.
//
always @ (posedge pclk or posedge rst)
    if (rst)
        sync_reg <= 0;
    else
        sync_reg <= scram_vid;

//
// Load enable generator
//
// This logic generates the parallel load enable input to the serializer
// register. Three cascaded flip-flops, clocked by the sclk, delay
// the leading edge of the word-rate clock. The first two flip-flops act as
// a dual-rank synchronizer to eliminate meta-stable conditions. The load enable
// to the serializer is asserted for the one clock cycle when the second
// flip-flop is asserted but the third is not.
//
always @ (posedge sclk or posedge rst)
    if (rst)
        ld_gen <= 3'b000;
    else
        ld_gen <= {ld_gen[1:0], pclk};

assign par2ser_ld = ld_gen[1] & ~ld_gen[2];

//
// Serializer register
//
// This register runs at the sclk rate. It loads in parallel when the load
// enable is asserted, otherwise it shifts MSB->LSB, two bits every sclk.
//

always @ (posedge sclk or posedge rst)
    if (rst)
        par2ser <= 0;
    else
        if (par2ser_ld)
            par2ser <= sync_reg;
        else
            par2ser <= {2'b00, par2ser[9:2]};

assign q[0] = par2ser[0];
assign q[1] = par2ser[1];

endmodule
