//------------------------------------------------------------------------------
// sdi_tx_top.v
//
// SMPTE 259M SDI top level Tx & Rx with pattern generator
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Oct. 28, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//          Oct 28, 2003        Release 1.0
//                                                          
// Other modules instanced in this design:
//          sdi_tx_ddr
//
//------------------------------------------------------------------------------
/*
Description of module:

This is the top level module for an SMPTE 259M SDI transmitter. This module 
instances all the IOBs and the DCM required to implement the SDI transmitter.
The actual SDI endcoder and serializer are implemented in the sdi_tx_ddr module.

This SDI transmitter uses DDR techniques to generate the 270Mbps bitstream. The
DCM takes in a 27MHz word-rate video clock and generates two phases of a 135MHz
clock. 135MHz is one-half the 270Mbps bit rate of SDI. The two 135MHz clock 
phases are 180 degrees apart in phase and are used to clock the DDR flip-flop
primitive FDDRRSE. The sdi_tx_ddr module produces two bits per cycle of the
135MHz clock. The FDDRRSE primitive serializes these two bits to produce a
270Mbps bitstream.

--------------------------------------------------------------------------------
*/

`timescale 1ns / 1 ps

module sdi_tx_top (
    clk27M,                 // 27MHz reference clock
    clkfb_in,               // DCM feedback clock input
    vid_in,                 // input video
    push_button1,           // resets all logic
    sdi_tx_dout_p,          // SDI-Tx differential data output
    sdi_tx_dout_n,
    clk_out
);

input           clk27M;
input           clkfb_in;
input   [9:0]   vid_in;
input           push_button1;       
output          sdi_tx_dout_p;
output          sdi_tx_dout_n;
output          clk_out;

//-----------------------------------------------------------------------------
// Signal definitions
//
wire        clk27M_int;         // clk27M input clock after IBUFG
wire        tx_sclk_p;          // CLKFX output of DCM after BUFG
wire        tx_sclk_n;          // CLKFX180 output of DCM after BUFG
wire        tx_pclk;            // CLK0 output of DCM after BUFG
wire [1:0]  tx_serout;          // serial output of SDI transmitter
wire        pb1;                // buffered push_button1 (reset)
wire        tx_dcm_locked;      // LOCKED output of DCM
wire        tx_ddr_out;         // serial output of the FDDRRSE primitive
wire        dcm_clk_fb;         // feedback clock for the DCM after going through IOB
wire        clk0_buf;           // CLK0 output of DCM
wire        clkfx_buf;          // CLKFX output of DCM
wire        clkfx180_buf;       // CLKFX180 output of DCM
wire [9:0]  video_in;           // parallel video input after IOBs
wire        VCC;
wire        GND;

//-----------------------------------------------------------------------------
// I/O Buffers
//

assign VCC = 1'b1;
assign GND = 1'b0;

//
// Instantiate the startup block
//
// The pushbutton input is used to issue a GSR reset to the FPGA.
//
STARTUP_VIRTEX2 STARTV2 (
    .CLK                (),
    .GSR                (~pb1),
    .GTS                ());
                                              
//
// Push button reset switch
//
IBUF_LVCMOS25 PB1 (
    .I      (push_button1), 
    .O      (pb1));

//
// 27MHz input clock buffer
//
IBUFG_LVCMOS25 CLK27 (
    .O      (clk27M_int),
    .I      (clk27M));

//
// This DCM produces a de-skewed 27MHz clock output (CLK0) and two phases of
// a 135MHz clock that are 180 degress out of phase.
//
DCM DCM1(
    .CLKIN      (clk27M_int),
    .CLKFB      (dcm_clk_fb),
    .RST        (GSR),
    .PSEN       (1'b0),
    .PSINCDEC   (1'b0),
    .PSCLK      (1'b0),
    .DSSEN      (1'b0),
    .CLK0       (clk0_buf),
    .CLK90      (),
    .CLK180     (),
    .CLK270     (),
    .CLKDV      (),
    .CLK2X      (),
    .CLK2X180   (),
    .CLKFX      (clkfx_buf),
    .CLKFX180   (clkfx180_buf),
    .STATUS     (),
    .LOCKED     (tx_dcm_locked),
    .PSDONE     ());
// synthesis attribute CLK_FEEDBACK of DCM1 is "1X"
// synthesis attribute CLKDV_DIVIDE of DCM1 is 2
// synthesis attribute CLKFX_DIVIDE of DCM1 is 1
// synthesis attribute CLKFX_MULTIPLY of DCM1 is 5
// synthesis attribute CLKIN_DIVIDE_BY_2 of DCM1 is "FALSE"
// synthesis attribute CLKIN_PERIOD of DCM1 is 37.037
// synthesis attribute CLKOUT_PHASE_SHIFT of DCM1 is "NONE"
// synthesis attribute DESKEW_ADJUST of DCM1 is "SYSTEM_SYNCHRONOUS"
// synthesis attribute DFS_FREQUENCY_MODE of DCM1 is "LOW"
// synthesis attribute DLL_FREQUENCY_MODE of DCM1 is "LOW"
// synthesis attribute DUTY_CYCLE_CORRECTION of DCM1 is "TRUE"
// synthesis attribute PHASE_SHIFT of DCM1 is 0
// synthesis attribute STARTUP_WAIT of DCM1 is "FALSE"
// synthesis translate_off
 defparam DCM1.CLK_FEEDBACK="1X";
 defparam DCM1.CLKDV_DIVIDE=2;
 defparam DCM1.CLKFX_DIVIDE=1;
 defparam DCM1.CLKFX_MULTIPLY=5;
 defparam DCM1.CLKIN_DIVIDE_BY_2="FALSE";
 defparam DCM1.CLKIN_PERIOD=27.778;
 defparam DCM1.CLKOUT_PHASE_SHIFT="NONE";
 defparam DCM1.DESKEW_ADJUST="SYSTEM_SYNCHRONOUS";
 defparam DCM1.DFS_FREQUENCY_MODE="LOW";
 defparam DCM1.DLL_FREQUENCY_MODE="LOW";
 defparam DCM1.DUTY_CYCLE_CORRECTION="TRUE";
 defparam DCM1.PHASE_SHIFT=0;
 defparam DCM1.STARTUP_WAIT="FALSE";
// synthesis translate_on

//
// This is the input buffer for the feedback clock for the DCM.
//
IBUFG CLKFB_IBUFG_INST(
    .I (clkfb_in),
    .O (dcm_clk_fb));

//
// This is the outpout buffer for the CLK0 output of the DCM. Normally this
// output would be wired to the clkfb_in pin of the FPGA with suitable PCB
// routing to match the routing delay of the video inputs.
//
OBUF_LVCMOS2 CLKOUT (
    .I (tx_pclk),
    .O (clk_out));

// 
// This is the BUFG for the CLK0 output of the DCM (27MHz).
//
BUFG CLK0_BUFG_INST(
    .I (clk0_buf),
    .O (tx_pclk));

//
// This is the BUFG for the CLKFX output of the DCM (135MHz).
//
BUFG CLKFX_BUFG_INST(
    .I (clkfx_buf),
    .O (tx_sclk_p));

//
// This is the BUFG for the CLKFX180 output of the DCM (135MHz).
//
BUFG CLKFX180_BUFG_INST(
    .I (clkfx180_buf),
    .O (tx_sclk_n));


//
// SDI transmitter
//
// The sdi_tx_ddr module contains the SDI encoder and the serializer. The
// serializer must produce two bit for every cycle of the 135MHz clock in
// order to interface with the DDR primitive.
//  
sdi_tx_ddr SDITX (
    .sclk           (tx_sclk_p),
    .pclk           (tx_pclk),
    .rst            (GSR),
    .vid_in         (video_in),
    .q              (tx_serout));

//
// SDI-Tx Port transmit data buffer
//
// This is the OBUF for for the SDI output bitstream. It uses an LVDS buffer
// to minimize jitter. This output is normally connected to a SDI compatible
// cable driver chip with LVDS inputs such as the National Semiconductor
// CLC001.
//
OBUFDS_LVDS_25 SDI_TXD (
    .I      (tx_ddr_out), 
    .O      (sdi_tx_dout_p), 
    .OB     (sdi_tx_dout_n));

//
// DDR primitive
// 
// The primitive contains two flip-flops and a MUX that convert the two bits
// from the sdi_tx_ddr module into a 270Mbps bitstream.
//
FDDRRSE DDR (
    .D0     (tx_serout[1]),
    .D1     (tx_serout[0]),
    .C0     (tx_sclk_p),
    .C1     (tx_sclk_n),
    .CE     (1'b1),
    .R      (1'b0),
    .S      (1'b0),
    .Q      (tx_ddr_out));
            

//
// Input video buffers
//
// These are the input buffers for the parallel video.
//
IBUF_LVCMOS25 VID0 (
    .O      (video_in[0]),
    .I      (vid_in[0]));

IBUF_LVCMOS25 VID1 (
    .O      (video_in[1]),
    .I      (vid_in[1]));

IBUF_LVCMOS25 VID2 (
    .O      (video_in[2]),
    .I      (vid_in[2]));

IBUF_LVCMOS25 VID3 (
    .O      (video_in[3]),
    .I      (vid_in[3]));

IBUF_LVCMOS25 VID4 (
    .O      (video_in[4]),
    .I      (vid_in[4]));

IBUF_LVCMOS25 VID5 (
    .O      (video_in[5]),
    .I      (vid_in[5]));

IBUF_LVCMOS25 VID6 (
    .O      (video_in[6]),
    .I      (vid_in[6]));

IBUF_LVCMOS25 VID7 (
    .O      (video_in[7]),
    .I      (vid_in[7]));

IBUF_LVCMOS25 VID8 (
    .O      (video_in[8]),
    .I      (vid_in[8]));

IBUF_LVCMOS25 VID9 (
    .O      (video_in[9]),
    .I      (vid_in[9]));


endmodule
