//------------------------------------------------------------------------------
// x247_rx_ext_cdr.v
//
// SMPTE 259M SDI Receiver using external CDR chip
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Oct. 28, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//              
//              Oct 28, 2003        Release 1.08
//                                          
// Other modules instanced in this design:
//      par_descrambler (from XAPP288)
//      par_framer_mult (from XAPP288)
//------------------------------------------------------------------------------
/*
Description of module:

This module is a SDI receiver that uses an external clock & data recovery chip
or "reclocker". This design has been tested with the National Semiconductor
CLC016 SDI CDR.

The CLC016 produces two differential outputs. One is the recoverd bit-rate
clock. The other is the original bitstream that has be synchronized to the
recovered clock. The serial clock is used by the FPGA to clock a shift
register to deserialize the bitstream and produce 10-bit words. The serial
clock is divided by 10 to get a word-rate clock. The word-rate clock drives the
descrambler and framer modules that take the encoded bitstream and produce
decoded and framed video words.

--------------------------------------------------------------------------------
*/

`timescale 1ns / 1 ps

module x247_rx_ext_cdr (
    mr_sd_rxc_p,            // MR-Rx SDI differential clock input from CLC016
    mr_sd_rxc_n,
    mr_sd_rxd_p,            // MR-Rx SDI differential data input from CLC016
    mr_sd_rxd_n,
    mr_sd_ser,              // asserted low when CLC016 is locked
    mr_sd_cd,               // carrier detect input from CLC016
    push_button1,           // resets all logic when low
    video_out,              // parallel video output
    trs,                    // asserted when receiver detects TRS symbols in bitstream
    video_clk               // recovered word-rate video clock output
);

input           mr_sd_rxc_p;
input           mr_sd_rxc_n;
input           mr_sd_rxd_p;
input           mr_sd_rxd_n;
input           mr_sd_ser;
input           mr_sd_cd;
input           push_button1;       
output  [9:0]   video_out;
output          trs;
output          video_clk;

//-----------------------------------------------------------------------------
// Signal definitions
//
reg     [4:0]           ringcntr;           // divide by 10 counter to generate rxrecclk
reg     [9:0]           ser2par_reg;        // deserialization register
reg     [9:0]           sync_reg;           // sync reg between clock domains
reg     [9:0]           desc_in_reg;        // input register to SMPTE descrambler
wire    [9:0]           desc_out;           // output of SMPTE descrambler
wire                    frame_en;           // framer enable
wire    [9:0]           framer_out;         // video out of framer
wire                    framer_nsp;         // framer new-start-position signal
reg     [9:0]           vid_reg;            // reg between framer and edh_processor
wire                    rx_sclk_int;        // receiver bit-rate clock after IOB
wire                    rx_pclk_int;        // receiver word-rate clock before BUFG
wire                    rx_sclk;            // global receiver bit-rate clock
wire                    rx_pclk;            // global receiver word-rate clock
wire                    rx_ser_dat;         // rx serial data input before input FF
reg                     rx_sdat;            // rx serial data input FF
wire                    clc_ser;            // indicates locked status of CDR chip
wire                    clc_cd;             // carrier detect signal from CDR chip
wire                    clc_locked;         // inverted version of clc_ser
wire                    rxrecclk;           // input to BUFG that drives rx_pclk
wire    [9:0]           rx_video;           // video from receiver
wire                    pb1;                // buffered push_button1 (reset)
reg                     sync_ld;            // loads the sync_reg
wire                    VCC;
wire                    GND;

//-----------------------------------------------------------------------------
// I/O Buffers
//

assign VCC = 1'b1;
assign GND = 1'b0;


//
// Instantiate the startup block
//
STARTUP_VIRTEX2 STARTV2 (
    .CLK                (),
    .GSR                (~pb1),
    .GTS                ());
                                              
//
// IOB for reset push button input
//
IBUF_LVCMOS25 PB1 (.I(push_button1), .O(pb1));


//
// Parallel video outputs
//
OBUF_LVCMOS25 VIDOUT_0 (.I(rx_video[0]), .O(video_out[0]));
OBUF_LVCMOS25 VIDOUT_1 (.I(rx_video[1]), .O(video_out[1]));
OBUF_LVCMOS25 VIDOUT_2 (.I(rx_video[2]), .O(video_out[2]));
OBUF_LVCMOS25 VIDOUT_3 (.I(rx_video[3]), .O(video_out[3]));
OBUF_LVCMOS25 VIDOUT_4 (.I(rx_video[4]), .O(video_out[4]));
OBUF_LVCMOS25 VIDOUT_5 (.I(rx_video[5]), .O(video_out[5]));
OBUF_LVCMOS25 VIDOUT_6 (.I(rx_video[6]), .O(video_out[6]));
OBUF_LVCMOS25 VIDOUT_7 (.I(rx_video[7]), .O(video_out[7]));
OBUF_LVCMOS25 VIDOUT_8 (.I(rx_video[8]), .O(video_out[8]));
OBUF_LVCMOS25 VIDOUT_9 (.I(rx_video[9]), .O(video_out[9]));


//
// Input buffers for the serial clock and data coming from CDR chip
//
// The IOBDELAY attribute is attached to the serial data input IOB. This
// adds the IOBDELAY element to the data path before the IOB flip-flop. This
// delay matches the global clock distribution delay that is added to the
// serial clock so that setup and hold time are met on the input to the
// IOB flip-flop.
//
IBUFDS_LVDSEXT_25_DCI MRSDCLK (
    .O      (rx_sclk_int),
    .I      (mr_sd_rxc_p),
    .IB     (mr_sd_rxc_n));

IBUFDS_LVDSEXT_25_DCI MRSDDAT (
    .O      (rx_ser_dat),
    .I      (mr_sd_rxd_p),
    .IB     (mr_sd_rxd_n));
// synthesis attribute IOBDELAY of MRSDDAT is IFD;

//
// Input FF for serial bit stream
//
// This FF is located in the IOB to minimize the delay time on the data
//
always @ (posedge rx_sclk)
    rx_sdat <= rx_ser_dat;
// synthesis attribute IOB of rx_sdat is "TRUE"


//
// Input buffer for the SER signal from CDR chip
//
// The SER output of the CLC016 will be low when the CLC016 is locked to the
// incoming bitstream. This can be used as an indicator to internal logic that
// the CDR is locked.
//
IBUF_LVDCI_25 MDSD_SER (
    .O      (clc_ser),
    .I      (mr_sd_ser));

assign clc_locked = ~clc_ser;

//
// Input buffer for the CD signal from the CDR chip
//
// The CD output of the CLC016 will be high when the CLC016 detects an input
// signal. It can be used as another status input to the logic in the FPGA.
//
IBUF_LVDCI_25 MDSD_CD (
    .O      (clc_cd),
    .I      (mr_sd_cd));

//
// Global clock buffers
//
// The rx_sclk is the globally distributed version of the 270MHz bit-rate 
// recovered clock from the CLC016 CDR. The rx_pclk is the global version of
// the word-rate clock that is generated by dividing rx_sclk by 10.
//
BUFG RXSCLK_BUF (
    .O      (rx_sclk),
    .I      (rx_sclk_int));


BUFG RXPCLK_BUF (
    .O      (rx_pclk), 
    .I      (rxrecclk));


//-----------------------------------------------------------------------------
// Receiver Section
//

//
// Rx parallel clock generation
//
// This code generates a 27MHz clock from the 270M from the CDR PLL. A 5-bit 
// ring counter is clocked by the 270M clock. The ring counter first fills up
// with all ones and then with all zeros, producing a 50% duty cycle 27MHz 
// clock.
//
always @ (posedge rx_sclk)
    ringcntr <= {ringcntr[3:0], ~ringcntr[4]};
// synthesis attribute INIT of ringcntr is 0 

assign rxrecclk = ringcntr[4];

//
// Deserialization and synchronization registers
//
// The ser2par_reg is a shift register that deserializes the data. The output
// of the ser2par_reg is loaded into the sync_reg at about the same time that
// as the falling edge of the rx_pclk. The data in the sync_reg is synchronized 
// to the rx_pclk domain by loading the contents of the sync_reg into the 
// desc_in_reg on the rising edge of rx_pclk. This gives us about 5 rx_sclk 
// cycles of setup time and hold time for the data going into the desc_in_reg, 
// minus clock distribution delays.
//
always @ (posedge rx_sclk)
    ser2par_reg <= {rx_sdat, ser2par_reg[9:1]};

always @ (posedge rx_sclk)
    if (ringcntr[3] & ~ringcntr[2])
        sync_ld <= 1'b1;
    else
        sync_ld <= 1'b0;

always @ (posedge rx_sclk)
    if (sync_ld)
        sync_reg <= ser2par_reg;

always @ (posedge rx_pclk)
    desc_in_reg <= sync_reg;

//
// par_descrambler
//
// The SDI parallel descrambler decodes the bitstream. This module can be found 
// in XAPP288.
//
par_descrambler dsc (
    .clk        (rx_pclk),
    .rst        (GSR),
    .ld         (VCC),
    .d          (desc_in_reg),
    .q          (desc_out));

//
// par_framer_mult
//
// The framer synchronizes to TRS symbols and then frames each 10-bit video word
// appropriately. There are two versions of the parallel framer available in the
// XAPP288 reference design. One uses a MULT18X18 primitive as a barrel shifter,
// the other version implements the barrel shifter in logic. The version that 
// uses the MULT18X18 primitive saves some CLB logic when a spare MULT18X18 
// is available in the FPGA.
//
par_framer_mult frm (
    .clk        (rx_pclk),
    .rst        (GSR),
    .ce         (VCC),
    .d          (desc_out),
    .frame_en   (frame_en),
    .q          (framer_out),
    .trs        (framer_trs),
    .nsp        (framer_nsp));

// if the EDH processor is being used, assign frame_en to framer_nsp | ep_sync_switch
// where ep_sync_switch is an output from the EDH processor that indicates when
// the video is in the synchronous switching interval

assign frame_en = framer_nsp  /*| ep_sync_switch*/;

//
// vid_reg
//
// Holds the parallel data out of framer
//
always @ (posedge rx_pclk)
    vid_reg <= framer_out;

assign rx_video = vid_reg;

//
// Output buffers for the video_clk and trs outputs
//
OBUF_LVCMOS25 VIDCLK_OUT (.I(rx_pclk),      .O(video_clk));
OBUF_LVCMOS25 TRS_OUT    (.I(framer_trs),   .O(trs));

endmodule
