//------------------------------------------------------------------------------
// x247_rx_int_cdr.v
//
// SMPTE 259M SDI receiver using XAPP250 for CDR
//
//
//
//                  Author: John F. Snow
//                  Staff Applications Engineer
//
//                  Video Applications
//                  Advanced Products Group
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Oct 28, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//      Oct 28, 2003        Release 1.0
//                                                  
// Other modules instanced in this design:
//      xapp250_cdr (XAPP250 CDR unit)
//      par_descrambler (from XAPP288)
//      par_framer (from XAPP288)
//
//------------------------------------------------------------------------------
/*
Module description:

This is the top level module for the XAPP247 SDI receiver using internal CDR. 
The CDR unit is based on XAPP250 and uses an inexpensive external 270MHz VCO
and loop filter combined with a phase detector built in the FPGA to implement
a PLL that locks to the frequency of the SDI bitstream. The CDR files from
XAPP250 are included in the reference design archive, but newer versions may
be available by downloading the XAPP250 reference designs.

After recovering the clock and data, the recovered data is deserialized, 
decoded, and framed. The SDI decoder and framer modules are found in XAPP288.
*/


`timescale 1ns / 1 ps

module x247_rx_int_cdr (
    clk54M_p,               // 54MHz clock
    clk54M_n,               //
    vco270M_p,              // 270MHz VCO clock
    vco270M_n,              // 270MHz VCO input buffer
    sd_rxd_p,               // SDI serial bit stream from equalizer
    sd_rxd_n,
    sd_rx_cd,               // carrier detect signal from external cable equalizer
    push_button1,           // resets all logic
    sdi_vco_up,             // 270MHz VCO faster
    sdi_vco_down,           // 270MHz VCO slower
    video_out,              // parallel video output
    video_clk,              // output 27MHz video clock
    trs                     // asserted when SDI TRS symbols are detected
);

input           clk54M_p;
input           clk54M_n;
input           vco270M_p;
input           vco270M_n;
input           sd_rxd_p;
input           sd_rxd_n;
input           sd_rx_cd;
input           push_button1;
output          sdi_vco_up;
output          sdi_vco_down;       
output  [9:0]   video_out;
output          video_clk;
output          trs;

//-----------------------------------------------------------------------------
// Signal definitions
//
wire        clk54M_ref;         // clk54M after IOB used as refclk to MGT
wire        vco270M_int;        // 270MHz VCO after after IOB
wire        rx_sclk;            // receiver bit-rate clock after BUFG
wire        rx_pclk;            // receiver word-rate clock after BUFG
wire        rx_ser_dat;         // rx serial data input before input FF
wire        rxrecclk;           // rx_sclk divided by 10
wire [9:0]  rx_video;           // video from receiver
reg  [4:0]  ringcntr;           // divide by 10 counter to generate rxrecclk
reg  [9:0]  ser2par_reg;        // deserialization register
reg  [9:0]  sync_reg;           // sync reg between clock domains
reg  [9:0]  desc_in_reg;        // input register to SMPTE descrambler
wire [9:0]  desc_out;           // output of SMPTE descrambler
wire        frame_en;           // framer enable
wire [9:0]  framer_out;         // video out of framer
wire        framer_nsp;         // framer new-start-position signal
wire        framer_trs;         // TRS output from framer
reg  [9:0]  vid_reg;            // output register
wire        pb1;                // buffered push_button1 (reset)
wire        eq_cd;              // sdi_rx_cd input after IOB
wire [1:0]  dataout;            // data from data recovery unit
wire        make_vco_faster;    // output of phase detector to VCO loop filter
wire        make_vco_slower;    // output of phase detector to VCO loop filter
reg         sync_ld;            // loads the sync_reg
wire        VCC;
wire        GND;

//-----------------------------------------------------------------------------
// I/O Buffers
//

assign VCC = 1'b1;
assign GND = 1'b0;

//
// Instantiate the startup block
//
STARTUP_VIRTEX2 STARTV2 (
    .CLK            (),
    .GSR            (~pb1),
    .GTS            ());

                                              
//
// IOB for reset push button input
//
IBUF_LVCMOS25 PB1 (.I(push_button1), .O(pb1));


//
// Parallel video outputs
//
OBUF_LVCMOS25 VIDOUT_0 (.I(rx_video[0]), .O(video_out[0]));
OBUF_LVCMOS25 VIDOUT_1 (.I(rx_video[1]), .O(video_out[1]));
OBUF_LVCMOS25 VIDOUT_2 (.I(rx_video[2]), .O(video_out[2]));
OBUF_LVCMOS25 VIDOUT_3 (.I(rx_video[3]), .O(video_out[3]));
OBUF_LVCMOS25 VIDOUT_4 (.I(rx_video[4]), .O(video_out[4]));
OBUF_LVCMOS25 VIDOUT_5 (.I(rx_video[5]), .O(video_out[5]));
OBUF_LVCMOS25 VIDOUT_6 (.I(rx_video[6]), .O(video_out[6]));
OBUF_LVCMOS25 VIDOUT_7 (.I(rx_video[7]), .O(video_out[7]));
OBUF_LVCMOS25 VIDOUT_8 (.I(rx_video[8]), .O(video_out[8]));
OBUF_LVCMOS25 VIDOUT_9 (.I(rx_video[9]), .O(video_out[9]));


//
// 54MHz input clock buffer
//
// The 54Mhz clock is used as a reference clock to the phase detector in the
// CDR unit. The phase detector will lock to this clock when not locked to the
// SDI bitstream. There is nothing special about using 54MHz. This just happens
// to be a clock value that was used on the evaluation board for this design.
// The phase detector divides this clock and the VCO output both down to a 
// common value of 1 MHz for comparison reasons. By changing the reference
// clock divider in the phase detector, difference reference clock frequencies
// can be used.
//
IBUFGDS_LVDSEXT_25_DCI CLK54 (
    .O      (clk54M_ref),
    .I      (clk54M_p),
    .IB     (clk54M_n));

//
// 270MHz VCO clock input
//
// This is the output of the external 270MHz VCO used as part of the CDR PLL.
//
IBUFGDS_LVDSEXT_25_DCI VCO270 (
    .O      (vco270M_int),
    .I      (vco270M_p),
    .IB     (vco270M_n));

//
// Input buffers for SDI bitstream
//
IBUFDS_LVDSEXT_25_DCI SDI_IN (
    .O      (rx_ser_dat),
    .I      (sd_rxd_p),
    .IB     (sd_rxd_n));

//
// Input buffer for the carrier detect signal from CLC014 equalizer
//
// In this design, the carrier detect output of the cable equalizer is used
// to force the CDR unit to lock to the reference clock rather than the 
// bitstream whenever the equalizer detects the absence of an SDI bitstream.
// This mechanism is currently used to resync the CDR unit in the absence of
// a bitstream because the phase detector currently lacks a lock detection
// circuit.
//
IBUF_LVCMOS25 CLC_CD (
    .O      (eq_cd),
    .I      (sd_rx_cd));

//
// Global clock buffers
//
BUFG RXSCLK_BUF (
    .O      (rx_sclk),
    .I      (vco270M_int));

BUFG RXPCLK_BUF (
    .O      (rx_pclk),
    .I      (rxrecclk));



//-----------------------------------------------------------------------------
// Receiver Section
//

//
// CDR
//
// This module implements the phase detector and data recovery unit for the
// SDI receiver. It uses the 54MHz clock as a reference clock. This is a very
// slightly modified version of the XAPP250 CDR unit that adds a force_resync
// input. This input is connected to the cable equalizer carrier detect
// signal to force the PLL to lock to the 54MHz reference whenever the carrier
// detect signal is negated.
// 
xapp250_cdr SDI_CDR (
    .vco            (rx_sclk),      
    .refclk         (clk54M_ref),
    .reset          (GSR),
    .data           (rx_ser_dat),
    .force_resync   (~eq_cd),
    .dataout        (dataout),
    .vcoisfast      (),
    .vcoisslow      (),
    .mkvcofaster    (make_vco_faster),
    .mkvcoslower    (make_vco_slower),
    .vco_tc         (),
    .refclk_tc      (),
    .phasedet       (),
    .clkisfastc2d   (),
    .clkisslowc2d   ()
);


//
// Rx parallel clock generation
//
// This code generates a 27MHz clock from the 270M from the CDR PLL. A 5-bit 
// ring counter is clocked by the 270M clock. The ring counter first fills up
// with all ones and then with all zeros, producing a 50% duty cycle 27MHz 
// clock.
//
always @ (posedge rx_sclk)
    ringcntr <= {ringcntr[3:0], ~ringcntr[4]};
// synthesis attribute init of ringcntr is 0 

assign rxrecclk = ringcntr[4];

//
// Deserialization and synchronization registers
//
// The ser2par_reg is a shift register that deserializes the data. The output
// of the ser2par_reg is loaded into the sync_reg at about the same time that
// as the falling edge of the rx_pclk. The data in the sync_reg is synchronized 
// to the rx_pclk domain by loading the contents of the sync_reg into the 
// desc_in_reg on the rising edge of rx_pclk. This gives us about 5 rx_sclk 
// cycles of setup time and hold time for the data going into the desc_in_reg, 
// minus clock distribution delays.
//
always @ (posedge rx_sclk)
    ser2par_reg <= {dataout[0], ser2par_reg[9:1]};

always @ (posedge rx_sclk)
    if (ringcntr[3] & ~ringcntr[2])
        sync_ld <= 1'b1;
    else
        sync_ld <= 1'b0;

always @ (posedge rx_sclk)
    if (sync_ld)
        sync_reg <= ser2par_reg;

always @ (posedge rx_pclk)
    desc_in_reg <= sync_reg;

//
// par_descrambler
//
// The SDI parallel descrambler decodes the bitstream. This module can be found 
// in XAPP288.
//
par_descrambler SDI_DESC (
    .clk        (rx_pclk),
    .rst        (GSR),
    .ld         (VCC),
    .d          (desc_in_reg),
    .q          (desc_out));

//
// par_framer_mult
//
// The framer synchronizes to TRS symbols and then frames each 10-bit video word
// appropriately. There are two versions of the parallel framer available in the
// XAPP288 reference design. One uses a MULT18X18 primitive as a barrel shifter,
// the other version implements the barrel shifter in logic. The version that 
// uses the MULT18X18 primitive saves some CLB logic when a spare MULT18X18 
// is available in the FPGA.
//
par_framer_mult SDI_FRM (
    .clk        (rx_pclk),
    .rst        (GSR),
    .ce         (VCC),
    .d          (desc_out),
    .frame_en   (frame_en),
    .q          (framer_out),
    .trs        (framer_trs),
    .nsp        (framer_nsp));

// if the EDH processor is being used, assign frame_en to framer_nsp | ep_sync_switch
// where ep_sync_switch is an output from the EDH processor that indicates when
// the video is in the synchronous switching interval
assign frame_en = framer_nsp  /*| ep_sync_switch*/;

//
// vid_reg
//
// Holds the parallel data out of framer
//
always @ (posedge rx_pclk)
    vid_reg <= framer_out;

assign rx_video = vid_reg;



//
// Output buffers for the video_clk and trs outputs
//
OBUF_LVCMOS25 VIDCLK_OUT (.I(rx_pclk),      .O(video_clk));
OBUF_LVCMOS25 TRS_OUT    (.I(framer_trs),   .O(trs));


//
// Output buffers for the loop filter control signals
// 
OBUF_LVDCI_33 SDVCO_U (.I(make_vco_faster), .O(sdi_vco_up));
OBUF_LVDCI_33 SDVCO_D (.I(make_vco_slower), .O(sdi_vco_down));


endmodule
