//------------------------------------------------------------------------------
// xapp250_cdr.v
//
// SDI CDR unit from XAPP250
//
//
//
//                  Author: Leonard Dieguez
//                  Xilinx, Inc.
//
//                  Copyright (c) 2003 Xilinx, Inc.
//                  All rights reserved
//
//                  Date:   Oct. 28, 2003
//
//                  RESTRICTED RIGHTS LEGEND
//
//      This software has not been published by the author, and 
//      has been disclosed to others for the purpose of enhancing 
//      and promoting design productivity in Xilinx products.
//
//      Therefore use, duplication or disclosure, now and in the 
//      future should give consideration to the productivity 
//      enhancements afforded the user of this code by the author's 
//      efforts.  Thank you for using our products !
//
// Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//              WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//              IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//              A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
// Revision:
//      Oct 28, 2003        Release 1.0
//                                                      
// Other modules instanced in this design:
//      pd_54to270
//      pd_clk2data22
//
//------------------------------------------------------------------------------
/*
Description of module:

This is the top level module of the XAPP250 CDR circuit. This module differs
slightly from the code in the XAPP250 reference design in that it adds a
force_resync input used to force the PLL to sync to the reference clock.

This module is designed to work with a 54MHz reference clock. If different
reference clock frequencies are used, then the pd_54to270 phase detector must
be modified accordingly. 
--------------------------------------------------------------------------------
*/
module  xapp250_cdr (
    // inputs
    vco,            // 270MHz VCO clock input
    refclk,         // 54MHz reference clock input
    reset,          // async reset
    data,           // serial data input
    force_resync,   // forces resync to refclk when high
    //outputs
    dataout,        // recovered data output
    vcoisfast,      // VCO is running fast
    vcoisslow,      // VCO is running slow
    mkvcofaster,    // output to loop filter to increase frequency of VCO
    mkvcoslower,    // output to loop filter to decrease frequency of VCO
    vco_tc,         // terminal count output of VCO counter
    refclk_tc,      // terminal count output of REFCLK counter
    //test
    phasedet,       // test output
    clklags,        // test output
    clkleads,       // test output
    clkisfastc2d,   // test output
    clkisslowc2d    // test output
);
    
input           vco;
input           refclk;
input           reset;
input           data;
input           force_resync;
output  [1:0]   dataout;
output          vcoisfast;
output          vcoisslow;
output          mkvcofaster;
output          mkvcoslower;
output          vco_tc;
output          refclk_tc;
output  [15:0]  phasedet;
output  [3:0]   clklags;
output  [3:0]   clkleads;
output          clkisfastc2d;
output          clkisslowc2d;

//-----------------------------------------------------------------------------
// Signal definitions
//
wire            up1 /* synthesis syn_keep=1 */;
wire            dn1 /* synthesis syn_keep=1 */;
wire            vco_tc;
wire            vcounlockedtrig;
wire            vcolockedcntTC;
reg             vcolockdet;
reg             vcolockdetR;
reg             vcolocked;
reg [15:0]      vcolockedcnt;
wire            pd_select;

//
// Trigger pulse for "un-locked detection
//
// If the clocks are not rolling ie. in locked  condition vcolockdet and 
// vcolockdetR should always be the same.
//
always @ (posedge refclk or posedge reset) begin 
    if (reset) begin
        vcolockdet  <= #1 1'b0 ;
        vcolockdetR <= #1 1'b0 ;
    end
    else begin
        vcolockdet  <= #1 vco_tc ;
        vcolockdetR <= #1 vcolockdet ;
    end
end

assign vcounlockedtrig = vcolockdetR ^ vcolockdet;



//
// Start a counter, reset if trig occurs. If the counter gets to 32K then 
// assumed in locked condition.once in locked condition do not allow counter 
// to count any more at this point try to lock to data.
//
always @ (posedge refclk or posedge reset ) begin 
    if (reset) begin
        vcolocked   <= #1 1'b0 ;
        vcolockedcnt    <= #1 16'h0000 ;
        end
    else if (vcounlockedtrig & ~vcolockedcntTC | force_resync) begin
        vcolocked   <= #1 1'b0 ;
        vcolockedcnt    <= #1 16'h0000 ;
        end
    else if (vcolockedcntTC) begin
        vcolocked   <= #1 1'b1 ;
        vcolockedcnt    <= #1 vcolockedcnt ;
        end
    else begin 
        vcolocked   <= #1 1'b0 ;
        vcolockedcnt    <= #1 vcolockedcnt + 1 ;
        end
end


assign vcolockedcntTC = vcolockedcnt[15];

assign pd_select = vcolocked & ~force_resync;

//
// phase detector
//
// This phase detector is used to lock the VCO to the 54MHz reference clock.
//
pd_54to270 pdref2vco (
    .vco        (vco),
    .refclk     (refclk),
    .reset      (reset),
    .vcoisfast  (vcoisfast),
    .vcoisslow  (vcoisslow),
    .vco_tc     (vco_tc),
    .refclk_tc  (refclk_tc)
);

//
// CDR
//
// This module contains the actual phase detector used to lock the VCO to the
// the bitstream and it also contains the data recovery unit.
//
pd_clk2data22 pd_clk2data (
    .clk            (~vco),         // run the module on the falling edge of the VCO
    .reset          (reset),
    .data           (data),         // data sampled with respect to the rising edge of the clock
    .data_out       (dataout),
    .mkclkfaster    (mkclkfaster1),
    .mkclkslower    (mkclkslower1),
    .phasedet_exp   (phasedet),
    .clklags_exp    (clklags),
    .clkleads_exp   (clkleads),
    .clkisfast_exp  (clkisfastc2d),
    .clkisslow_exp  (clkisslowc2d));


//
// These MUXes are determine which phase detector is actively driving the
// control signals out to the loop filter.
//
MUXF5 mvfast (.I0(vcoisslow), .I1(mkclkfaster1), .S(pd_select), .O(mkvcofaster));
MUXF5 mvslow (.I0(vcoisfast), .I1(mkclkslower1), .S(pd_select), .O(mkvcoslower));

endmodule